import re
from typing import List, MutableMapping, Optional

from feast.core.Feature_pb2 import FeatureSpec as FeatureSpecProto
from feast.serving.ServingService_pb2 import FeatureReference as FeatureReferenceProto
from feast.types import Value_pb2 as ValueTypeProto
from feast.value_type import ValueType


class Feature:
    """Feature field type"""

    def __init__(
        self,
        name: str,
        dtype: ValueType,
        labels: Optional[MutableMapping[str, str]] = None,
    ):
        self._name = name
        if not isinstance(dtype, ValueType):
            raise ValueError("dtype is not a valid ValueType")
        self._dtype = dtype
        if labels is None:
            self._labels = dict()  # type: MutableMapping
        else:
            self._labels = labels

    def __eq__(self, other):
        if (
            self.name != other.name
            or self.dtype != other.dtype
            or self.labels != other.labels
        ):
            return False
        return True

    def __lt__(self, other):
        return self.name < other.name

    @property
    def name(self):
        """
        Getter for name of this field
        """
        return self._name

    @property
    def dtype(self) -> ValueType:
        """
        Getter for data type of this field
        """
        return self._dtype

    @property
    def labels(self) -> MutableMapping[str, str]:
        """
        Getter for labels of this field
        """
        return self._labels

    def to_proto(self) -> FeatureSpecProto:
        """Converts Feature object to its Protocol Buffer representation"""
        value_type = ValueTypeProto.ValueType.Enum.Value(self.dtype.name)

        return FeatureSpecProto(
            name=self.name,
            value_type=value_type,
            labels=self.labels,
        )

    @classmethod
    def from_proto(cls, feature_proto: FeatureSpecProto):
        """
        Args:
            feature_proto: FeatureSpecV2 protobuf object
        Returns:
            Feature object
        """

        feature = cls(
            name=feature_proto.name,
            dtype=ValueType(feature_proto.value_type),
            labels=feature_proto.labels,
        )

        return feature


def build_feature_references(
    feature_ref_strs: List[str],
) -> List[FeatureReferenceProto]:
    """
    Builds a list of FeatureReference protos from a list of FeatureReference strings
    Args:
        feature_ref_strs: List of string feature references
    Returns:
        A list of FeatureReference protos parsed from args.
    """

    valid_feature_ref_regex = re.compile("[^:]+:[^:]+")
    for ref_str in feature_ref_strs:
        if not re.fullmatch(valid_feature_ref_regex, ref_str):
            raise ValueError(
                "Feature reference should be in the form  of <feature table>:<feature name>"
            )

    return [
        FeatureReferenceProto(
            feature_table=ref_str.split(":")[0], name=ref_str.split(":")[-1]
        )
        for ref_str in feature_ref_strs
    ]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/types/Field.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from feast.types import Value_pb2 as feast_dot_types_dot_Value__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17\x66\x65\x61st/types/Field.proto\x12\x0b\x66\x65\x61st.types\x1a\x17\x66\x65\x61st/types/Value.proto\"8\n\x05\x46ield\x12\x0c\n\x04name\x18\x01 \x01(\t\x12!\n\x05value\x18\x02 \x01(\x0b\x32\x12.feast.types.ValueBx\n\x1f\x64\x65v.caraml.store.protobuf.typesB\nFieldProtoZIgithub.com/caraml-dev/caraml-store/caraml-store-sdk/go/protos/feast/typesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'feast.types.Field_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\037dev.caraml.store.protobuf.typesB\nFieldProtoZIgithub.com/caraml-dev/caraml-store/caraml-store-sdk/go/protos/feast/types'
  _FIELD._serialized_start=65
  _FIELD._serialized_end=121
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/core/SparkOverride.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1e\x66\x65\x61st/core/SparkOverride.proto\x12\nfeast.core\"i\n\rSparkOverride\x12\x12\n\ndriver_cpu\x18\x01 \x01(\x05\x12\x15\n\rdriver_memory\x18\x02 \x01(\t\x12\x14\n\x0c\x65xecutor_cpu\x18\x03 \x01(\x05\x12\x17\n\x0f\x65xecutor_memory\x18\x04 \x01(\tB~\n\x1e\x64\x65v.caraml.store.protobuf.coreB\x12SparkOverrideProtoZHgithub.com/caraml-dev/caraml-store/caraml-store-sdk/go/protos/feast/coreb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'feast.core.SparkOverride_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\036dev.caraml.store.protobuf.coreB\022SparkOverrideProtoZHgithub.com/caraml-dev/caraml-store/caraml-store-sdk/go/protos/feast/core'
  _SPARKOVERRIDE._serialized_start=46
  _SPARKOVERRIDE._serialized_end=151
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import feast.types.Value_pb2
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class Field(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    name: builtins.str
    @property
    def value(self) -> feast.types.Value_pb2.Value: ...
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        value: feast.types.Value_pb2.Value | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["name", b"name", "value", b"value"]) -> None: ...

global___Field = Field



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import feast.core.DataSource_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.timestamp_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _JobType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _JobTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_JobType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    INVALID_JOB: _JobType.ValueType  # 0
    BATCH_INGESTION_JOB: _JobType.ValueType  # 1
    STREAM_INGESTION_JOB: _JobType.ValueType  # 2
    RETRIEVAL_JOB: _JobType.ValueType  # 4

class JobType(_JobType, metaclass=_JobTypeEnumTypeWrapper): ...

INVALID_JOB: JobType.ValueType  # 0
BATCH_INGESTION_JOB: JobType.ValueType  # 1
STREAM_INGESTION_JOB: JobType.ValueType  # 2
RETRIEVAL_JOB: JobType.ValueType  # 4
global___JobType = JobType

class _JobStatus:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _JobStatusEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_JobStatus.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    JOB_STATUS_INVALID: _JobStatus.ValueType  # 0
    JOB_STATUS_PENDING: _JobStatus.ValueType  # 1
    """The Job has be registered and waiting to get scheduled to run"""
    JOB_STATUS_RUNNING: _JobStatus.ValueType  # 2
    """The Job is currently processing its task"""
    JOB_STATUS_DONE: _JobStatus.ValueType  # 3
    """The Job has successfully completed its task"""
    JOB_STATUS_ERROR: _JobStatus.ValueType  # 4
    """The Job has encountered an error while processing its task"""

class JobStatus(_JobStatus, metaclass=_JobStatusEnumTypeWrapper): ...

JOB_STATUS_INVALID: JobStatus.ValueType  # 0
JOB_STATUS_PENDING: JobStatus.ValueType  # 1
"""The Job has be registered and waiting to get scheduled to run"""
JOB_STATUS_RUNNING: JobStatus.ValueType  # 2
"""The Job is currently processing its task"""
JOB_STATUS_DONE: JobStatus.ValueType  # 3
"""The Job has successfully completed its task"""
JOB_STATUS_ERROR: JobStatus.ValueType  # 4
"""The Job has encountered an error while processing its task"""
global___JobStatus = JobStatus

@typing_extensions.final
class ScheduledJob(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    TABLE_NAME_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    INGESTION_TIMESPAN_FIELD_NUMBER: builtins.int
    CRON_SCHEDULE_FIELD_NUMBER: builtins.int
    id: builtins.str
    """Identifier of the Job"""
    table_name: builtins.str
    project: builtins.str
    ingestion_timespan: builtins.int
    """Timespan of the ingested data per job, in days. The data from  end of the day - timespan till end of the day will be ingested. Eg. if the job execution date is 10/4/2021, and ingestion timespan is 2, then data from 9/4/2021 00:00 to 10/4/2021 23:59 (inclusive) will be ingested."""
    cron_schedule: builtins.str
    """Crontab string. Eg. 0 13 * * *"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        table_name: builtins.str = ...,
        project: builtins.str = ...,
        ingestion_timespan: builtins.int = ...,
        cron_schedule: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["cron_schedule", b"cron_schedule", "id", b"id", "ingestion_timespan", b"ingestion_timespan", "project", b"project", "table_name", b"table_name"]) -> None: ...

global___ScheduledJob = ScheduledJob

@typing_extensions.final
class Job(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class RetrievalJobMeta(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        OUTPUT_LOCATION_FIELD_NUMBER: builtins.int
        output_location: builtins.str
        def __init__(
            self,
            *,
            output_location: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["output_location", b"output_location"]) -> None: ...

    @typing_extensions.final
    class OfflineToOnlineMeta(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TABLE_NAME_FIELD_NUMBER: builtins.int
        table_name: builtins.str
        def __init__(
            self,
            *,
            table_name: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["table_name", b"table_name"]) -> None: ...

    @typing_extensions.final
    class StreamToOnlineMeta(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TABLE_NAME_FIELD_NUMBER: builtins.int
        table_name: builtins.str
        def __init__(
            self,
            *,
            table_name: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["table_name", b"table_name"]) -> None: ...

    ID_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    HASH_FIELD_NUMBER: builtins.int
    START_TIME_FIELD_NUMBER: builtins.int
    RETRIEVAL_FIELD_NUMBER: builtins.int
    BATCH_INGESTION_FIELD_NUMBER: builtins.int
    STREAM_INGESTION_FIELD_NUMBER: builtins.int
    LOG_URI_FIELD_NUMBER: builtins.int
    ERROR_MESSAGE_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    id: builtins.str
    """Identifier of the Job"""
    type: global___JobType.ValueType
    """Type of the Job"""
    status: global___JobStatus.ValueType
    """Current job status"""
    hash: builtins.str
    """Deterministic hash of the Job"""
    @property
    def start_time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Start time of the Job"""
    @property
    def retrieval(self) -> global___Job.RetrievalJobMeta: ...
    @property
    def batch_ingestion(self) -> global___Job.OfflineToOnlineMeta: ...
    @property
    def stream_ingestion(self) -> global___Job.StreamToOnlineMeta: ...
    log_uri: builtins.str
    """Path to Spark job logs, if available"""
    error_message: builtins.str
    """Spark job error message, if available"""
    project: builtins.str
    """Project"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        type: global___JobType.ValueType = ...,
        status: global___JobStatus.ValueType = ...,
        hash: builtins.str = ...,
        start_time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        retrieval: global___Job.RetrievalJobMeta | None = ...,
        batch_ingestion: global___Job.OfflineToOnlineMeta | None = ...,
        stream_ingestion: global___Job.StreamToOnlineMeta | None = ...,
        log_uri: builtins.str = ...,
        error_message: builtins.str = ...,
        project: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["batch_ingestion", b"batch_ingestion", "meta", b"meta", "retrieval", b"retrieval", "start_time", b"start_time", "stream_ingestion", b"stream_ingestion"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["batch_ingestion", b"batch_ingestion", "error_message", b"error_message", "hash", b"hash", "id", b"id", "log_uri", b"log_uri", "meta", b"meta", "project", b"project", "retrieval", b"retrieval", "start_time", b"start_time", "status", b"status", "stream_ingestion", b"stream_ingestion", "type", b"type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["meta", b"meta"]) -> typing_extensions.Literal["retrieval", "batch_ingestion", "stream_ingestion"] | None: ...

global___Job = Job

@typing_extensions.final
class StartOfflineToOnlineIngestionJobRequest(google.protobuf.message.Message):
    """Ingest data from offline store into online store"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_FIELD_NUMBER: builtins.int
    TABLE_NAME_FIELD_NUMBER: builtins.int
    START_DATE_FIELD_NUMBER: builtins.int
    END_DATE_FIELD_NUMBER: builtins.int
    DELTA_INGESTION_FIELD_NUMBER: builtins.int
    project: builtins.str
    """Feature table to ingest"""
    table_name: builtins.str
    @property
    def start_date(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Start of time range for source data from offline store"""
    @property
    def end_date(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """End of time range for source data from offline store"""
    delta_ingestion: builtins.bool
    """optional setting for delta ingestion"""
    def __init__(
        self,
        *,
        project: builtins.str = ...,
        table_name: builtins.str = ...,
        start_date: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        end_date: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        delta_ingestion: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["end_date", b"end_date", "start_date", b"start_date"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["delta_ingestion", b"delta_ingestion", "end_date", b"end_date", "project", b"project", "start_date", b"start_date", "table_name", b"table_name"]) -> None: ...

global___StartOfflineToOnlineIngestionJobRequest = StartOfflineToOnlineIngestionJobRequest

@typing_extensions.final
class StartOfflineToOnlineIngestionJobResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    JOB_START_TIME_FIELD_NUMBER: builtins.int
    TABLE_NAME_FIELD_NUMBER: builtins.int
    LOG_URI_FIELD_NUMBER: builtins.int
    id: builtins.str
    """Job ID assigned by Feast"""
    @property
    def job_start_time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Job start time"""
    table_name: builtins.str
    """Feature table associated with the job"""
    log_uri: builtins.str
    """Path to Spark job logs, if available"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        job_start_time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        table_name: builtins.str = ...,
        log_uri: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["job_start_time", b"job_start_time"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["id", b"id", "job_start_time", b"job_start_time", "log_uri", b"log_uri", "table_name", b"table_name"]) -> None: ...

global___StartOfflineToOnlineIngestionJobResponse = StartOfflineToOnlineIngestionJobResponse

@typing_extensions.final
class StartStreamIngestionJobRequest(google.protobuf.message.Message):
    """Ingest data from streaming source into online store"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_FIELD_NUMBER: builtins.int
    TABLE_NAME_FIELD_NUMBER: builtins.int
    project: builtins.str
    """Feature table to ingest"""
    table_name: builtins.str
    def __init__(
        self,
        *,
        project: builtins.str = ...,
        table_name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["project", b"project", "table_name", b"table_name"]) -> None: ...

global___StartStreamIngestionJobRequest = StartStreamIngestionJobRequest

@typing_extensions.final
class StartStreamIngestionJobResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    id: builtins.str
    """Job ID assigned by Feast"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["id", b"id"]) -> None: ...

global___StartStreamIngestionJobResponse = StartStreamIngestionJobResponse

@typing_extensions.final
class ScheduleOfflineToOnlineIngestionJobRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_FIELD_NUMBER: builtins.int
    TABLE_NAME_FIELD_NUMBER: builtins.int
    INGESTION_TIMESPAN_FIELD_NUMBER: builtins.int
    CRON_SCHEDULE_FIELD_NUMBER: builtins.int
    project: builtins.str
    """Feature table to ingest"""
    table_name: builtins.str
    ingestion_timespan: builtins.int
    """Timespan of the ingested data per job, in days. The data from  end of the day - timespan till end of the day will be ingested. Eg. if the job execution date is 10/4/2021, and ingestion timespan is 2, then data from 9/4/2021 00:00 to 10/4/2021 23:59 (inclusive) will be ingested."""
    cron_schedule: builtins.str
    """Crontab string. Eg. 0 13 * * *"""
    def __init__(
        self,
        *,
        project: builtins.str = ...,
        table_name: builtins.str = ...,
        ingestion_timespan: builtins.int = ...,
        cron_schedule: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["cron_schedule", b"cron_schedule", "ingestion_timespan", b"ingestion_timespan", "project", b"project", "table_name", b"table_name"]) -> None: ...

global___ScheduleOfflineToOnlineIngestionJobRequest = ScheduleOfflineToOnlineIngestionJobRequest

@typing_extensions.final
class ScheduleOfflineToOnlineIngestionJobResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ScheduleOfflineToOnlineIngestionJobResponse = ScheduleOfflineToOnlineIngestionJobResponse

@typing_extensions.final
class UnscheduleOfflineToOnlineIngestionJobRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_FIELD_NUMBER: builtins.int
    TABLE_NAME_FIELD_NUMBER: builtins.int
    project: builtins.str
    table_name: builtins.str
    def __init__(
        self,
        *,
        project: builtins.str = ...,
        table_name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["project", b"project", "table_name", b"table_name"]) -> None: ...

global___UnscheduleOfflineToOnlineIngestionJobRequest = UnscheduleOfflineToOnlineIngestionJobRequest

@typing_extensions.final
class UnscheduleOfflineToOnlineIngestionJobResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___UnscheduleOfflineToOnlineIngestionJobResponse = UnscheduleOfflineToOnlineIngestionJobResponse

@typing_extensions.final
class GetHistoricalFeaturesRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURE_REFS_FIELD_NUMBER: builtins.int
    ENTITY_SOURCE_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    OUTPUT_LOCATION_FIELD_NUMBER: builtins.int
    OUTPUT_FORMAT_FIELD_NUMBER: builtins.int
    @property
    def feature_refs(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """List of feature references that are being retrieved"""
    @property
    def entity_source(self) -> feast.core.DataSource_pb2.DataSource:
        """Batch DataSource that can be used to obtain entity values for historical retrieval.
        For each entity value, a feature value will be retrieved for that value/timestamp
        Only 'BATCH_*' source types are supported.
        Currently only BATCH_FILE source type is supported.
        """
    project: builtins.str
    """Optional field to specify project name override. If specified, uses the
    given project for retrieval. Overrides the projects specified in
    Feature References if both are specified.
    """
    output_location: builtins.str
    """Specifies the path in a bucket to write the exported feature data files
    Export to AWS S3 - s3://path/to/features
    Export to GCP GCS -  gs://path/to/features
    """
    output_format: builtins.str
    """Specify format name for output, eg. parquet"""
    def __init__(
        self,
        *,
        feature_refs: collections.abc.Iterable[builtins.str] | None = ...,
        entity_source: feast.core.DataSource_pb2.DataSource | None = ...,
        project: builtins.str = ...,
        output_location: builtins.str = ...,
        output_format: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["entity_source", b"entity_source"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["entity_source", b"entity_source", "feature_refs", b"feature_refs", "output_format", b"output_format", "output_location", b"output_location", "project", b"project"]) -> None: ...

global___GetHistoricalFeaturesRequest = GetHistoricalFeaturesRequest

@typing_extensions.final
class GetHistoricalFeaturesResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    OUTPUT_FILE_URI_FIELD_NUMBER: builtins.int
    JOB_START_TIME_FIELD_NUMBER: builtins.int
    LOG_URI_FIELD_NUMBER: builtins.int
    id: builtins.str
    """Export Job with ID assigned by Feast"""
    output_file_uri: builtins.str
    """Uri to the join result output file"""
    @property
    def job_start_time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Job start time"""
    log_uri: builtins.str
    """Path to Spark job logs, if available"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        output_file_uri: builtins.str = ...,
        job_start_time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        log_uri: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["job_start_time", b"job_start_time"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["id", b"id", "job_start_time", b"job_start_time", "log_uri", b"log_uri", "output_file_uri", b"output_file_uri"]) -> None: ...

global___GetHistoricalFeaturesResponse = GetHistoricalFeaturesResponse

@typing_extensions.final
class ListJobsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INCLUDE_TERMINATED_FIELD_NUMBER: builtins.int
    TABLE_NAME_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    include_terminated: builtins.bool
    table_name: builtins.str
    project: builtins.str
    type: global___JobType.ValueType
    def __init__(
        self,
        *,
        include_terminated: builtins.bool = ...,
        table_name: builtins.str = ...,
        project: builtins.str = ...,
        type: global___JobType.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["include_terminated", b"include_terminated", "project", b"project", "table_name", b"table_name", "type", b"type"]) -> None: ...

global___ListJobsRequest = ListJobsRequest

@typing_extensions.final
class ListScheduledJobsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_FIELD_NUMBER: builtins.int
    TABLE_NAME_FIELD_NUMBER: builtins.int
    project: builtins.str
    table_name: builtins.str
    def __init__(
        self,
        *,
        project: builtins.str = ...,
        table_name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["project", b"project", "table_name", b"table_name"]) -> None: ...

global___ListScheduledJobsRequest = ListScheduledJobsRequest

@typing_extensions.final
class ListJobsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    JOBS_FIELD_NUMBER: builtins.int
    @property
    def jobs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Job]: ...
    def __init__(
        self,
        *,
        jobs: collections.abc.Iterable[global___Job] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["jobs", b"jobs"]) -> None: ...

global___ListJobsResponse = ListJobsResponse

@typing_extensions.final
class ListScheduledJobsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    JOBS_FIELD_NUMBER: builtins.int
    @property
    def jobs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ScheduledJob]: ...
    def __init__(
        self,
        *,
        jobs: collections.abc.Iterable[global___ScheduledJob] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["jobs", b"jobs"]) -> None: ...

global___ListScheduledJobsResponse = ListScheduledJobsResponse

@typing_extensions.final
class GetJobRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    JOB_ID_FIELD_NUMBER: builtins.int
    job_id: builtins.str
    def __init__(
        self,
        *,
        job_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["job_id", b"job_id"]) -> None: ...

global___GetJobRequest = GetJobRequest

@typing_extensions.final
class GetJobResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    JOB_FIELD_NUMBER: builtins.int
    @property
    def job(self) -> global___Job: ...
    def __init__(
        self,
        *,
        job: global___Job | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["job", b"job"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["job", b"job"]) -> None: ...

global___GetJobResponse = GetJobResponse

@typing_extensions.final
class CancelJobRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    JOB_ID_FIELD_NUMBER: builtins.int
    job_id: builtins.str
    def __init__(
        self,
        *,
        job_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["job_id", b"job_id"]) -> None: ...

global___CancelJobRequest = CancelJobRequest

@typing_extensions.final
class CancelJobResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___CancelJobResponse = CancelJobResponse

@typing_extensions.final
class UnscheduleJobRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    JOB_ID_FIELD_NUMBER: builtins.int
    job_id: builtins.str
    def __init__(
        self,
        *,
        job_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["job_id", b"job_id"]) -> None: ...

global___UnscheduleJobRequest = UnscheduleJobRequest

@typing_extensions.final
class UnscheduleJobResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___UnscheduleJobResponse = UnscheduleJobResponse

@typing_extensions.final
class GetHealthMetricsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_FIELD_NUMBER: builtins.int
    TABLE_NAMES_FIELD_NUMBER: builtins.int
    project: builtins.str
    @property
    def table_names(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        project: builtins.str = ...,
        table_names: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["project", b"project", "table_names", b"table_names"]) -> None: ...

global___GetHealthMetricsRequest = GetHealthMetricsRequest

@typing_extensions.final
class GetHealthMetricsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PASSED_FIELD_NUMBER: builtins.int
    FAILED_FIELD_NUMBER: builtins.int
    @property
    def passed(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    @property
    def failed(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        passed: collections.abc.Iterable[builtins.str] | None = ...,
        failed: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["failed", b"failed", "passed", b"passed"]) -> None: ...

global___GetHealthMetricsResponse = GetHealthMetricsResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class SparkOverride(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DRIVER_CPU_FIELD_NUMBER: builtins.int
    DRIVER_MEMORY_FIELD_NUMBER: builtins.int
    EXECUTOR_CPU_FIELD_NUMBER: builtins.int
    EXECUTOR_MEMORY_FIELD_NUMBER: builtins.int
    driver_cpu: builtins.int
    """Override CPU requested for driver"""
    driver_memory: builtins.str
    """Override memory requested for driver"""
    executor_cpu: builtins.int
    """Override CPU requested for executor"""
    executor_memory: builtins.str
    """Override memory requested for executor"""
    def __init__(
        self,
        *,
        driver_cpu: builtins.int = ...,
        driver_memory: builtins.str = ...,
        executor_cpu: builtins.int = ...,
        executor_memory: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["driver_cpu", b"driver_cpu", "driver_memory", b"driver_memory", "executor_cpu", b"executor_cpu", "executor_memory", b"executor_memory"]) -> None: ...

global___SparkOverride = SparkOverride



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/core/CoreService.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from feast.core import Entity_pb2 as feast_dot_core_dot_Entity__pb2
from feast.core import Feature_pb2 as feast_dot_core_dot_Feature__pb2
from feast.core import FeatureTable_pb2 as feast_dot_core_dot_FeatureTable__pb2
from feast.core import OnlineStore_pb2 as feast_dot_core_dot_OnlineStore__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1c\x66\x65\x61st/core/CoreService.proto\x12\nfeast.core\x1a\x17\x66\x65\x61st/core/Entity.proto\x1a\x18\x66\x65\x61st/core/Feature.proto\x1a\x1d\x66\x65\x61st/core/FeatureTable.proto\x1a\x1c\x66\x65\x61st/core/OnlineStore.proto\"1\n\x10GetEntityRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\"7\n\x11GetEntityResponse\x12\"\n\x06\x65ntity\x18\x01 \x01(\x0b\x32\x12.feast.core.Entity\"\xdc\x01\n\x13ListEntitiesRequest\x12\x36\n\x06\x66ilter\x18\x01 \x01(\x0b\x32&.feast.core.ListEntitiesRequest.Filter\x1a\x8c\x01\n\x06\x46ilter\x12\x0f\n\x07project\x18\x03 \x01(\t\x12\x42\n\x06labels\x18\x04 \x03(\x0b\x32\x32.feast.core.ListEntitiesRequest.Filter.LabelsEntry\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"<\n\x14ListEntitiesResponse\x12$\n\x08\x65ntities\x18\x01 \x03(\x0b\x32\x12.feast.core.Entity\"\xee\x01\n\x13ListFeaturesRequest\x12\x36\n\x06\x66ilter\x18\x01 \x01(\x0b\x32&.feast.core.ListFeaturesRequest.Filter\x1a\x9e\x01\n\x06\x46ilter\x12\x42\n\x06labels\x18\x01 \x03(\x0b\x32\x32.feast.core.ListFeaturesRequest.Filter.LabelsEntry\x12\x10\n\x08\x65ntities\x18\x02 \x03(\t\x12\x0f\n\x07project\x18\x03 \x01(\t\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xa8\x01\n\x14ListFeaturesResponse\x12@\n\x08\x66\x65\x61tures\x18\x02 \x03(\x0b\x32..feast.core.ListFeaturesResponse.FeaturesEntry\x1aH\n\rFeaturesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12&\n\x05value\x18\x02 \x01(\x0b\x32\x17.feast.core.FeatureSpec:\x02\x38\x01J\x04\x08\x01\x10\x02\"K\n\x12\x41pplyEntityRequest\x12$\n\x04spec\x18\x01 \x01(\x0b\x32\x16.feast.core.EntitySpec\x12\x0f\n\x07project\x18\x02 \x01(\t\"9\n\x13\x41pplyEntityResponse\x12\"\n\x06\x65ntity\x18\x01 \x01(\x0b\x32\x12.feast.core.Entity\"\x1c\n\x1aGetFeastCoreVersionRequest\".\n\x1bGetFeastCoreVersionResponse\x12\x0f\n\x07version\x18\x01 \x01(\t\"$\n\x14\x43reateProjectRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"\x17\n\x15\x43reateProjectResponse\"%\n\x15\x41rchiveProjectRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"\x18\n\x16\x41rchiveProjectResponse\"\x15\n\x13ListProjectsRequest\"(\n\x14ListProjectsResponse\x12\x10\n\x08projects\x18\x01 \x03(\t\"]\n\x18\x41pplyFeatureTableRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x30\n\ntable_spec\x18\x02 \x01(\x0b\x32\x1c.feast.core.FeatureTableSpec\"D\n\x19\x41pplyFeatureTableResponse\x12\'\n\x05table\x18\x01 \x01(\x0b\x32\x18.feast.core.FeatureTable\"7\n\x16GetFeatureTableRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\"B\n\x17GetFeatureTableResponse\x12\'\n\x05table\x18\x01 \x01(\x0b\x32\x18.feast.core.FeatureTable\"\xeb\x01\n\x18ListFeatureTablesRequest\x12;\n\x06\x66ilter\x18\x01 \x01(\x0b\x32+.feast.core.ListFeatureTablesRequest.Filter\x1a\x91\x01\n\x06\x46ilter\x12\x0f\n\x07project\x18\x01 \x01(\t\x12G\n\x06labels\x18\x03 \x03(\x0b\x32\x37.feast.core.ListFeatureTablesRequest.Filter.LabelsEntry\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"E\n\x19ListFeatureTablesResponse\x12(\n\x06tables\x18\x01 \x03(\x0b\x32\x18.feast.core.FeatureTable\":\n\x19\x44\x65leteFeatureTableRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\"\x1c\n\x1a\x44\x65leteFeatureTableResponse\"\x19\n\x17ListOnlineStoresRequest\"I\n\x18ListOnlineStoresResponse\x12-\n\x0conline_store\x18\x01 \x03(\x0b\x32\x17.feast.core.OnlineStore\"%\n\x15GetOnlineStoreRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"\xa6\x01\n\x16GetOnlineStoreResponse\x12-\n\x0conline_store\x18\x01 \x01(\x0b\x32\x17.feast.core.OnlineStore\x12\x39\n\x06status\x18\x02 \x01(\x0e\x32).feast.core.GetOnlineStoreResponse.Status\"\"\n\x06Status\x12\n\n\x06\x41\x43TIVE\x10\x00\x12\x0c\n\x08\x41RCHIVED\x10\x01\"K\n\x1aRegisterOnlineStoreRequest\x12-\n\x0conline_store\x18\x01 \x01(\x0b\x32\x17.feast.core.OnlineStore\"\xc2\x01\n\x1bRegisterOnlineStoreResponse\x12-\n\x0conline_store\x18\x01 \x01(\x0b\x32\x17.feast.core.OnlineStore\x12>\n\x06status\x18\x02 \x01(\x0e\x32..feast.core.RegisterOnlineStoreResponse.Status\"4\n\x06Status\x12\r\n\tNO_CHANGE\x10\x00\x12\x0e\n\nREGISTERED\x10\x01\x12\x0b\n\x07UPDATED\x10\x02\")\n\x19\x41rchiveOnlineStoreRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"\x1c\n\x1a\x41rchiveOnlineStoreResponse2\xc1\x0b\n\x0b\x43oreService\x12\x66\n\x13GetFeastCoreVersion\x12&.feast.core.GetFeastCoreVersionRequest\x1a\'.feast.core.GetFeastCoreVersionResponse\x12H\n\tGetEntity\x12\x1c.feast.core.GetEntityRequest\x1a\x1d.feast.core.GetEntityResponse\x12Q\n\x0cListFeatures\x12\x1f.feast.core.ListFeaturesRequest\x1a .feast.core.ListFeaturesResponse\x12N\n\x0b\x41pplyEntity\x12\x1e.feast.core.ApplyEntityRequest\x1a\x1f.feast.core.ApplyEntityResponse\x12Q\n\x0cListEntities\x12\x1f.feast.core.ListEntitiesRequest\x1a .feast.core.ListEntitiesResponse\x12T\n\rCreateProject\x12 .feast.core.CreateProjectRequest\x1a!.feast.core.CreateProjectResponse\x12W\n\x0e\x41rchiveProject\x12!.feast.core.ArchiveProjectRequest\x1a\".feast.core.ArchiveProjectResponse\x12Q\n\x0cListProjects\x12\x1f.feast.core.ListProjectsRequest\x1a .feast.core.ListProjectsResponse\x12`\n\x11\x41pplyFeatureTable\x12$.feast.core.ApplyFeatureTableRequest\x1a%.feast.core.ApplyFeatureTableResponse\x12`\n\x11ListFeatureTables\x12$.feast.core.ListFeatureTablesRequest\x1a%.feast.core.ListFeatureTablesResponse\x12Z\n\x0fGetFeatureTable\x12\".feast.core.GetFeatureTableRequest\x1a#.feast.core.GetFeatureTableResponse\x12\x63\n\x12\x44\x65leteFeatureTable\x12%.feast.core.DeleteFeatureTableRequest\x1a&.feast.core.DeleteFeatureTableResponse\x12]\n\x10ListOnlineStores\x12#.feast.core.ListOnlineStoresRequest\x1a$.feast.core.ListOnlineStoresResponse\x12W\n\x0eGetOnlineStore\x12!.feast.core.GetOnlineStoreRequest\x1a\".feast.core.GetOnlineStoreResponse\x12\x66\n\x13RegisterOnlineStore\x12&.feast.core.RegisterOnlineStoreRequest\x1a\'.feast.core.RegisterOnlineStoreResponse\x12\x63\n\x12\x41rchiveOnlineStore\x12%.feast.core.ArchiveOnlineStoreRequest\x1a&.feast.core.ArchiveOnlineStoreResponseB|\n\x1e\x64\x65v.caraml.store.protobuf.coreB\x10\x43oreServiceProtoZHgithub.com/caraml-dev/caraml-store/caraml-store-sdk/go/protos/feast/coreb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'feast.core.CoreService_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\036dev.caraml.store.protobuf.coreB\020CoreServiceProtoZHgithub.com/caraml-dev/caraml-store/caraml-store-sdk/go/protos/feast/core'
  _LISTENTITIESREQUEST_FILTER_LABELSENTRY._options = None
  _LISTENTITIESREQUEST_FILTER_LABELSENTRY._serialized_options = b'8\001'
  _LISTFEATURESREQUEST_FILTER_LABELSENTRY._options = None
  _LISTFEATURESREQUEST_FILTER_LABELSENTRY._serialized_options = b'8\001'
  _LISTFEATURESRESPONSE_FEATURESENTRY._options = None
  _LISTFEATURESRESPONSE_FEATURESENTRY._serialized_options = b'8\001'
  _LISTFEATURETABLESREQUEST_FILTER_LABELSENTRY._options = None
  _LISTFEATURETABLESREQUEST_FILTER_LABELSENTRY._serialized_options = b'8\001'
  _GETENTITYREQUEST._serialized_start=156
  _GETENTITYREQUEST._serialized_end=205
  _GETENTITYRESPONSE._serialized_start=207
  _GETENTITYRESPONSE._serialized_end=262
  _LISTENTITIESREQUEST._serialized_start=265
  _LISTENTITIESREQUEST._serialized_end=485
  _LISTENTITIESREQUEST_FILTER._serialized_start=345
  _LISTENTITIESREQUEST_FILTER._serialized_end=485
  _LISTENTITIESREQUEST_FILTER_LABELSENTRY._serialized_start=440
  _LISTENTITIESREQUEST_FILTER_LABELSENTRY._serialized_end=485
  _LISTENTITIESRESPONSE._serialized_start=487
  _LISTENTITIESRESPONSE._serialized_end=547
  _LISTFEATURESREQUEST._serialized_start=550
  _LISTFEATURESREQUEST._serialized_end=788
  _LISTFEATURESREQUEST_FILTER._serialized_start=630
  _LISTFEATURESREQUEST_FILTER._serialized_end=788
  _LISTFEATURESREQUEST_FILTER_LABELSENTRY._serialized_start=440
  _LISTFEATURESREQUEST_FILTER_LABELSENTRY._serialized_end=485
  _LISTFEATURESRESPONSE._serialized_start=791
  _LISTFEATURESRESPONSE._serialized_end=959
  _LISTFEATURESRESPONSE_FEATURESENTRY._serialized_start=881
  _LISTFEATURESRESPONSE_FEATURESENTRY._serialized_end=953
  _APPLYENTITYREQUEST._serialized_start=961
  _APPLYENTITYREQUEST._serialized_end=1036
  _APPLYENTITYRESPONSE._serialized_start=1038
  _APPLYENTITYRESPONSE._serialized_end=1095
  _GETFEASTCOREVERSIONREQUEST._serialized_start=1097
  _GETFEASTCOREVERSIONREQUEST._serialized_end=1125
  _GETFEASTCOREVERSIONRESPONSE._serialized_start=1127
  _GETFEASTCOREVERSIONRESPONSE._serialized_end=1173
  _CREATEPROJECTREQUEST._serialized_start=1175
  _CREATEPROJECTREQUEST._serialized_end=1211
  _CREATEPROJECTRESPONSE._serialized_start=1213
  _CREATEPROJECTRESPONSE._serialized_end=1236
  _ARCHIVEPROJECTREQUEST._serialized_start=1238
  _ARCHIVEPROJECTREQUEST._serialized_end=1275
  _ARCHIVEPROJECTRESPONSE._serialized_start=1277
  _ARCHIVEPROJECTRESPONSE._serialized_end=1301
  _LISTPROJECTSREQUEST._serialized_start=1303
  _LISTPROJECTSREQUEST._serialized_end=1324
  _LISTPROJECTSRESPONSE._serialized_start=1326
  _LISTPROJECTSRESPONSE._serialized_end=1366
  _APPLYFEATURETABLEREQUEST._serialized_start=1368
  _APPLYFEATURETABLEREQUEST._serialized_end=1461
  _APPLYFEATURETABLERESPONSE._serialized_start=1463
  _APPLYFEATURETABLERESPONSE._serialized_end=1531
  _GETFEATURETABLEREQUEST._serialized_start=1533
  _GETFEATURETABLEREQUEST._serialized_end=1588
  _GETFEATURETABLERESPONSE._serialized_start=1590
  _GETFEATURETABLERESPONSE._serialized_end=1656
  _LISTFEATURETABLESREQUEST._serialized_start=1659
  _LISTFEATURETABLESREQUEST._serialized_end=1894
  _LISTFEATURETABLESREQUEST_FILTER._serialized_start=1749
  _LISTFEATURETABLESREQUEST_FILTER._serialized_end=1894
  _LISTFEATURETABLESREQUEST_FILTER_LABELSENTRY._serialized_start=440
  _LISTFEATURETABLESREQUEST_FILTER_LABELSENTRY._serialized_end=485
  _LISTFEATURETABLESRESPONSE._serialized_start=1896
  _LISTFEATURETABLESRESPONSE._serialized_end=1965
  _DELETEFEATURETABLEREQUEST._serialized_start=1967
  _DELETEFEATURETABLEREQUEST._serialized_end=2025
  _DELETEFEATURETABLERESPONSE._serialized_start=2027
  _DELETEFEATURETABLERESPONSE._serialized_end=2055
  _LISTONLINESTORESREQUEST._serialized_start=2057
  _LISTONLINESTORESREQUEST._serialized_end=2082
  _LISTONLINESTORESRESPONSE._serialized_start=2084
  _LISTONLINESTORESRESPONSE._serialized_end=2157
  _GETONLINESTOREREQUEST._serialized_start=2159
  _GETONLINESTOREREQUEST._serialized_end=2196
  _GETONLINESTORERESPONSE._serialized_start=2199
  _GETONLINESTORERESPONSE._serialized_end=2365
  _GETONLINESTORERESPONSE_STATUS._serialized_start=2331
  _GETONLINESTORERESPONSE_STATUS._serialized_end=2365
  _REGISTERONLINESTOREREQUEST._serialized_start=2367
  _REGISTERONLINESTOREREQUEST._serialized_end=2442
  _REGISTERONLINESTORERESPONSE._serialized_start=2445
  _REGISTERONLINESTORERESPONSE._serialized_end=2639
  _REGISTERONLINESTORERESPONSE_STATUS._serialized_start=2587
  _REGISTERONLINESTORERESPONSE_STATUS._serialized_end=2639
  _ARCHIVEONLINESTOREREQUEST._serialized_start=2641
  _ARCHIVEONLINESTOREREQUEST._serialized_end=2682
  _ARCHIVEONLINESTORERESPONSE._serialized_start=2684
  _ARCHIVEONLINESTORERESPONSE._serialized_end=2712
  _CORESERVICE._serialized_start=2715
  _CORESERVICE._serialized_end=4188
# @@protoc_insertion_point(module_scope)



from typing import List, Dict, Any, cast, Type, Union

from google.protobuf.json_format import MessageToDict

from feast.serving.ServingService_pb2 import (
    GetOnlineFeaturesRequest,
    GetOnlineFeaturesResponse,
)
from feast.types.Value_pb2 import Value, ValueType


class OnlineResponse:
    """
    Defines a online response in feast.
    """

    def __init__(self, online_response_proto: GetOnlineFeaturesResponse):
        """
        Construct a native online response from its protobuf version.
        Args:
        online_response_proto: GetOnlineResponse proto object to construct from.
        """
        self.proto = online_response_proto

    @property
    def rows(self):
        """
        Getter for GetOnlineResponse's field_values.
        """
        return self.proto.results

    def to_dict(self) -> Dict[str, Any]:
        """
        Converts GetOnlineFeaturesResponse features into a dictionary form.
        """
        field_names = [field_name for field_name in self.proto.metadata.field_names.val]
        features_dict: Dict[str, List[Any]] = {k: list() for k in field_names}

        for row in self.rows:
            for field_name, value in zip(field_names, row.values):
                native_type_value = _feast_value_type_to_python_type(value)
                features_dict[field_name].append(native_type_value)
        return features_dict


def infer_online_entity_rows(
    entity_rows: List[Dict[str, Any]]
) -> List[GetOnlineFeaturesRequest.EntityRow]:
    """
    Builds a list of EntityRow protos from Python native type format passed by user.
    Args:
        entity_rows: A list of dictionaries where each key-value is an entity-name, entity-value pair.
    Returns:
        A list of EntityRow protos parsed from args.
    """

    entity_rows_dicts = cast(List[Dict[str, Any]], entity_rows)
    entity_row_list = []
    entity_type_map = dict()

    for entity in entity_rows_dicts:
        fields = {}
        for key, value in entity.items():
            # Allow for feast.types.Value
            if isinstance(value, Value):
                proto_value = value
            else:
                # Infer the specific type for this row
                current_dtype = _python_type_to_entity_value_type(type(value))

                if key not in entity_type_map:
                    entity_type_map[key] = current_dtype
                else:
                    if current_dtype != entity_type_map[key]:
                        raise TypeError(
                            f"Input entity {key} has mixed types, {current_dtype} and {entity_type_map[key]}. That is not allowed. "
                        )
                proto_value = _python_value_to_entity_value(current_dtype, value)
            fields[key] = proto_value
        entity_row_list.append(GetOnlineFeaturesRequest.EntityRow(fields=fields))
    return entity_row_list


def _python_type_to_entity_value_type(python_type: Type) -> ValueType:
    """
    Finds the equivalent Feast Value Type for a Python value. Only types applicable to an Entity are supported here.
    Args:
        value: Python primitive type
    Returns:
        Feast Value Type
    """

    type_map: Dict[Type, ValueType] = {
        int: ValueType.INT64,
        str: ValueType.STRING,
        bool: ValueType.BOOL,
    }
    return type_map[python_type]


def _python_value_to_entity_value(value_type: ValueType, value):
    if value_type == ValueType.INT64:
        return Value(int64_val=value)
    elif value_type == ValueType.BOOL:
        return Value(bool_val=value)
    else:
        return Value(string_val=value)


def _feast_value_type_to_python_type(field_value_proto: Value) -> Any:
    """
    Converts field value Proto to Dict and returns each field's Feast Value Type value
    in their respective Python value.
    Args:
        field_value_proto: Field value Proto
    Returns:
        Python native type representation/version of the given field_value_proto
    """
    field_value_dict = MessageToDict(field_value_proto)

    for k, v in field_value_dict.items():
        if k == "int64Val":
            return int(v)
        if k == "bytesVal":
            return bytes(v)
        if (k == "int64ListVal") or (k == "int32ListVal"):
            return [int(item) for item in v["val"]]
        if (k == "floatListVal") or (k == "doubleListVal"):
            return [float(item) for item in v["val"]]
        if k == "stringListVal":
            return [str(item) for item in v["val"]]
        if k == "bytesListVal":
            return [bytes(item) for item in v["val"]]
        if k == "boolListVal":
            return [bool(item) for item in v["val"]]

        if k in ["int32Val", "floatVal", "doubleVal", "stringVal", "boolVal"]:
            return v
        else:
            raise TypeError(
                f"Casting to Python native type for type {k} failed. "
                f"Type {k} not found"
            )



import yaml


def yaml_loader(yml, load_single=False):
    """
    Loads one or more Feast resources from a YAML path or string. Multiple resources
    can be divided by three hyphens '---'
    Args:
        yml: A path ending in .yaml or .yml, or a YAML string
        load_single: Expect only a single YAML resource, fail otherwise
    Returns:
        Either a single YAML dictionary or a list of YAML dictionaries
    """

    yml_content = _get_yaml_contents(yml)
    yaml_strings = yml_content.strip("---").split("---")

    # Return a single resource dict
    if load_single:
        if len(yaml_strings) > 1:
            raise Exception(
                f"More than one YAML file is being loaded when only a single file is supported: ${yaml_strings}"
            )
        return _yaml_to_dict(yaml_strings[0])

    # Return a list of resource dicts
    resources = []
    for yaml_string in yaml_strings:
        resources.append(_yaml_to_dict(yaml_string))
    return resources


def _get_yaml_contents(yml: str) -> str:
    """
    Returns the YAML contents from an object. If a path ending with .yaml or
    .yml is passed, it will be read for its contents. If a string containing
    YAML is passed, that will be returned.
    Args:
        yml: Path of YAML file or string containing YAML
    Returns:
        String object containing YAML
    """
    if (
        isinstance(yml, str)
        and yml.count("\n") == 0
        and (".yaml" in yml.lower() or ".yml" in yml.lower())
    ):
        with open(yml, "r") as f:
            yml_content = f.read()

    elif isinstance(yml, str):
        yml_content = yml
    else:
        raise Exception(
            f"Invalid YAML provided. Please provide either a file path or YAML string.\n"
            f"Provided YAML: {yml}"
        )
    return yml_content


def _yaml_to_dict(yaml_string):
    """
    Converts a yaml string to dictionary
    Args:
        yaml_string: String containing YAML
    Returns:
        Dictionary containing the same object
    """

    return yaml.safe_load(yaml_string)



from google.cloud import bigquery
import pandas as pd
import uuid

from datetime import datetime, timedelta

from feast.data_source import BigQuerySource


def stage_entities_to_bq(
    entity_source: pd.DataFrame, project: str, dataset: str
) -> BigQuerySource:
    """
    Stores given (entity) dataframe as new table in BQ. Name of the table generated based on current time.
    Table will expire in 1 day.
    Returns BigQuerySource with reference to created table.
    """

    bq_client: bigquery.Client = bigquery.Client()
    destination = bigquery.TableReference(
        bigquery.DatasetReference(project, dataset),
        f"_entities_{uuid.uuid4()}",
    )

    # prevent casting ns -> ms exception inside pyarrow
    entity_source["event_timestamp"] = entity_source["event_timestamp"].dt.floor("ms")

    load_job_config = bigquery.LoadJobConfig(
        time_partitioning=bigquery.TimePartitioning(
            type_=bigquery.TimePartitioningType.DAY,
            field="event_timestamp",
        )
    )
    load_job: bigquery.LoadJob = bq_client.load_table_from_dataframe(
        entity_source,
        destination,
        job_config=load_job_config,
    )
    load_job.result()  # wait until complete

    dest_table: bigquery.Table = bq_client.get_table(destination)
    dest_table.expires = datetime.now() + timedelta(days=1)
    bq_client.update_table(dest_table, fields=["expires"])

    return BigQuerySource(
        event_timestamp_column="event_timestamp",
        table_ref=f"{destination.project}:{destination.dataset_id}.{destination.table_id}",
    )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/core/DataSource.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from feast.core import DataFormat_pb2 as feast_dot_core_dot_DataFormat__pb2
from feast.core import SparkOverride_pb2 as feast_dot_core_dot_SparkOverride__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x66\x65\x61st/core/DataSource.proto\x12\nfeast.core\x1a\x1b\x66\x65\x61st/core/DataFormat.proto\x1a\x1e\x66\x65\x61st/core/SparkOverride.proto\"\xa8\x07\n\nDataSource\x12/\n\x04type\x18\x01 \x01(\x0e\x32!.feast.core.DataSource.SourceType\x12?\n\rfield_mapping\x18\x02 \x03(\x0b\x32(.feast.core.DataSource.FieldMappingEntry\x12\x1e\n\x16\x65vent_timestamp_column\x18\x03 \x01(\t\x12\x1d\n\x15\x64\x61te_partition_column\x18\x04 \x01(\t\x12 \n\x18\x63reated_timestamp_column\x18\x05 \x01(\t\x12:\n\x0c\x66ile_options\x18\x0b \x01(\x0b\x32\".feast.core.DataSource.FileOptionsH\x00\x12\x42\n\x10\x62igquery_options\x18\x0c \x01(\x0b\x32&.feast.core.DataSource.BigQueryOptionsH\x00\x12<\n\rkafka_options\x18\r \x01(\x0b\x32#.feast.core.DataSource.KafkaOptionsH\x00\x1a\x33\n\x11\x46ieldMappingEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x7f\n\x0b\x46ileOptions\x12+\n\x0b\x66ile_format\x18\x01 \x01(\x0b\x32\x16.feast.core.FileFormat\x12\x10\n\x08\x66ile_url\x18\x02 \x01(\t\x12\x31\n\x0espark_override\x18\x03 \x01(\x0b\x32\x19.feast.core.SparkOverride\x1aW\n\x0f\x42igQueryOptions\x12\x11\n\ttable_ref\x18\x01 \x01(\t\x12\x31\n\x0espark_override\x18\x02 \x01(\x0b\x32\x19.feast.core.SparkOverride\x1a\x9d\x01\n\x0cKafkaOptions\x12\x19\n\x11\x62ootstrap_servers\x18\x01 \x01(\t\x12\r\n\x05topic\x18\x02 \x01(\t\x12\x30\n\x0emessage_format\x18\x03 \x01(\x0b\x32\x18.feast.core.StreamFormat\x12\x31\n\x0espark_override\x18\x04 \x01(\x0b\x32\x19.feast.core.SparkOverride\"O\n\nSourceType\x12\x0b\n\x07INVALID\x10\x00\x12\x0e\n\nBATCH_FILE\x10\x01\x12\x12\n\x0e\x42\x41TCH_BIGQUERY\x10\x02\x12\x10\n\x0cSTREAM_KAFKA\x10\x03\x42\t\n\x07optionsB{\n\x1e\x64\x65v.caraml.store.protobuf.coreB\x0f\x44\x61taSourceProtoZHgithub.com/caraml-dev/caraml-store/caraml-store-sdk/go/protos/feast/coreb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'feast.core.DataSource_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\036dev.caraml.store.protobuf.coreB\017DataSourceProtoZHgithub.com/caraml-dev/caraml-store/caraml-store-sdk/go/protos/feast/core'
  _DATASOURCE_FIELDMAPPINGENTRY._options = None
  _DATASOURCE_FIELDMAPPINGENTRY._serialized_options = b'8\001'
  _DATASOURCE._serialized_start=105
  _DATASOURCE._serialized_end=1041
  _DATASOURCE_FIELDMAPPINGENTRY._serialized_start=520
  _DATASOURCE_FIELDMAPPINGENTRY._serialized_end=571
  _DATASOURCE_FILEOPTIONS._serialized_start=573
  _DATASOURCE_FILEOPTIONS._serialized_end=700
  _DATASOURCE_BIGQUERYOPTIONS._serialized_start=702
  _DATASOURCE_BIGQUERYOPTIONS._serialized_end=789
  _DATASOURCE_KAFKAOPTIONS._serialized_start=792
  _DATASOURCE_KAFKAOPTIONS._serialized_end=949
  _DATASOURCE_SOURCETYPE._serialized_start=951
  _DATASOURCE_SOURCETYPE._serialized_end=1030
# @@protoc_insertion_point(module_scope)



from typing import Any, Dict, List, Union, Optional
from datetime import datetime
from dataclasses import dataclass

import grpc
from croniter import croniter

from feast.core.CoreService_pb2_grpc import CoreServiceStub
from feast.core.CoreService_pb2 import (
    ListOnlineStoresRequest,
    ListOnlineStoresResponse,
    GetFeatureTableRequest,
    ApplyFeatureTableRequest,
    ListProjectsRequest,
    ListFeatureTablesRequest,
    DeleteFeatureTableRequest,
    GetEntityRequest,
    ListEntitiesRequest,
    ApplyEntityRequest,
)
from feast.data_source import FileSource, BigQuerySource
from feast.entity import Entity
from feast.feature import build_feature_references
from feast.feature_table import FeatureTable
from feast.online_response import infer_online_entity_rows, OnlineResponse
from feast.serving.ServingService_pb2_grpc import ServingServiceStub
from feast.serving.ServingService_pb2 import GetOnlineFeaturesRequest
from feast_spark.api.JobService_pb2 import (
    StartOfflineToOnlineIngestionJobRequest,
    StartOfflineToOnlineIngestionJobResponse,
    GetHistoricalFeaturesRequest,
    GetHistoricalFeaturesResponse,
    GetJobRequest,
    ListJobsRequest,
    CancelJobRequest,
    UnscheduleJobRequest,
    Job,
    ScheduleOfflineToOnlineIngestionJobRequest, ScheduledJob, ListScheduledJobsRequest, StartStreamIngestionJobResponse,
    StartStreamIngestionJobRequest, JobType, INVALID_JOB,
)
from feast_spark.api.JobService_pb2_grpc import JobServiceStub


@dataclass
class Client:
    registry_url: Optional[str] = None
    serving_url: Optional[str] = None
    _core_service_stub: Optional[CoreServiceStub] = None
    _serving_service_stub: Optional[ServingServiceStub] = None
    _job_service_stub: Optional[JobServiceStub] = None

    @property
    def _core_service(self):
        """
        Creates or returns the gRPC Feast Core Service Stub

        Returns: CoreServiceStub
        """
        if not self.registry_url:
            raise ValueError("registry_url has not been set")

        if not self._core_service_stub:
            channel = grpc.insecure_channel(self.registry_url)
            self._core_service_stub = CoreServiceStub(channel)
        return self._core_service_stub

    @property
    def _job_service(self):
        """
        Creates or returns the gRPC Feast Job Service Stub

        Returns: JobServiceStub
        """
        if not self.registry_url:
            raise ValueError("registry_url has not been set")

        if not self._job_service_stub:
            channel = grpc.insecure_channel(self.registry_url)
            self._job_service_stub = JobServiceStub(channel)
        return self._job_service_stub

    @property
    def _serving_service(self):
        """
        Creates or returns the gRPC Feast Serving Service Stub

        Returns: ServingServiceStub
        """
        if not self.serving_url:
            raise ValueError("serving_url has not been set")

        if not self._serving_service_stub:
            channel = grpc.insecure_channel(self.serving_url)
            self._serving_service_stub = ServingServiceStub(channel)
        return self._serving_service_stub

    def list_projects(self) -> List[str]:
        """
        List all active Feast projects
        Returns:
            List of project names
        """

        response = self._core_service.ListProjects(
            ListProjectsRequest(),
        )
        return list(response.projects)

    def apply_entity(self, entity: Entity, project: str):
        """
        Registers a single entity with Feast
        Args:
            entity: Entity that will be registered
            project: Project name
        """

        entity.is_valid()
        entity_proto = entity.to_spec_proto()

        # Convert the entity to a request and send to Feast Core
        apply_entity_response = self._core_service.ApplyEntity(
            ApplyEntityRequest(project=project, spec=entity_proto),
        )

        # Extract the returned entity
        applied_entity = Entity.from_proto(apply_entity_response.entity)

        # Deep copy from the returned entity to the local entity
        entity._update_from_entity(applied_entity)

    def list_entities(
        self, project: str = "", labels: Optional[Dict[str, str]] = None
    ) -> List[Entity]:
        """
        Retrieve a list of entities from Feast Core
        Args:
            project: Filter entities based on project name
            labels: User-defined labels that these entities are associated with
        Returns:
            List of entities
        """

        filter = ListEntitiesRequest.Filter(project=project, labels=labels or dict())

        # Get latest entities from Feast Core
        entity_protos = self._core_service.ListEntities(
            ListEntitiesRequest(filter=filter),
        )

        # Extract entities and return
        entities = []
        for entity_proto in entity_protos.entities:
            entity = Entity.from_proto(entity_proto)
            entity._client = self
            entities.append(entity)
        return entities

    def get_entity(self, name: str, project: str) -> Entity:
        """
        Retrieves an entity.
        Args:
            project: Feast project that this entity belongs to
            name: Name of entity
        Returns:
            Returns either the specified entity, or raises an exception if
            none is found
        """

        get_entity_response = self._core_service.GetEntity(
            GetEntityRequest(project=project, name=name.strip()),
        )
        entity = Entity.from_proto(get_entity_response.entity)

        return entity

    def list_feature_tables(
        self, project: str, labels: Optional[Dict[str, str]] = None
    ) -> List[FeatureTable]:
        """
        Retrieve a list of feature tables from Feast Core
        Args:
            project: Filter feature tables based on project name
            labels: Filter by feature table labels
        Returns:
            List of feature tables
        """

        # Get latest feature tables from Feast Core
        feature_table_protos = self._core_service.ListFeatureTables(
            ListFeatureTablesRequest(
                filter=ListFeatureTablesRequest.Filter(
                    project=project, labels=labels or dict()
                )
            )
        )

        # Extract feature tables and return
        feature_tables = []
        for feature_table_proto in feature_table_protos.tables:
            feature_table = FeatureTable.from_proto(feature_table_proto)
            feature_table._client = self
            feature_tables.append(feature_table)
        return feature_tables

    def get_feature_table(self, name: str, project: str) -> FeatureTable:
        """
        Retrieves a feature table.

        Args:
            name: Name of feature table
            project: Feast project that this feature table belongs to

        Returns:
            Returns either the requested feature table spec or raises an exception if
            none is found
        """
        response = self._core_service.GetFeatureTable(
            GetFeatureTableRequest(project=project, name=name.strip()),
        )
        return FeatureTable.from_proto(response.table)

    def apply_feature_table(self, table: FeatureTable, project: str):
        """
        Apply a feature table.

        Args:
            table: Feature table spec
            project: Feast project that this feature table belongs to
        """
        self._core_service.ApplyFeatureTable(
            ApplyFeatureTableRequest(project=project, table_spec=table.to_spec_proto())
        )

    def list_online_stores(self) -> ListOnlineStoresResponse:
        """
        List online stores
        Returns: ListOnlineStoresResponse
        """
        return self._core_service.ListOnlineStore(ListOnlineStoresRequest())

    def get_online_features(
        self,
        feature_refs: List[str],
        entity_rows: List[Dict[str, Any]],
        project,
    ) -> OnlineResponse:
        """
        Retrieves the latest online feature data from Feast Serving.
        Args:
            feature_refs: List of feature references that will be returned for each entity.
                Each feature reference should have the following format:
                "feature_table:feature" where "feature_table" & "feature" refer to
                the feature and feature table names respectively.
                Only the feature name is required.
            entity_rows: A list of dictionaries where each key-value is an entity-name, entity-value pair.
            project: Feast project which the features belong to
        Returns:
            OnlineResponse containing the feature data in records.
            Each EntityRow provided will yield one record, which contains
            data fields with data value and field status metadata (if included)
        """
        return OnlineResponse(
            self._serving_service.GetOnlineFeatures(
                GetOnlineFeaturesRequest(
                    features=build_feature_references(feature_ref_strs=feature_refs),
                    entity_rows=infer_online_entity_rows(entity_rows),
                    project=project,
                )
            )
        )

    def start_offline_to_online_ingestion(
        self,
        feature_table: str,
        start: datetime,
        end: datetime,
        project: str,
        delta_ingestion: bool = False,
    ) -> StartOfflineToOnlineIngestionJobResponse:
        """
        Start offline to online ingestion job
        Args:
            feature_table: feature table name
            start: start datetime to filter records to be ingested
            end: end datetime to filter records to be ingested
            project: Feast project name
            delta_ingestion: boolean setting for delta ingestion

        Returns: StartOfflineToOnlineIngestionJobResponse
        """
        request = StartOfflineToOnlineIngestionJobRequest(
            project=project, table_name=feature_table, delta_ingestion=delta_ingestion
        )
        request.start_date.FromDatetime(start)
        request.end_date.FromDatetime(end)
        return self._job_service.StartOfflineToOnlineIngestionJob(request)

    def start_stream_ingestion(
        self,
        feature_table: str,
        project: str
    ) -> StartStreamIngestionJobResponse:
        """
        Starts or update a stream ingestion job for the given feature table
        Args:
            feature_table: feature table name
            project: Feast project name

        Returns: StartStreamIngestionJobResponse
        """
        request = StartStreamIngestionJobRequest(
            project=project, table_name=feature_table
        )
        return self._job_service.StartStreamIngestionJob(request)

    def schedule_offline_to_online_ingestion(
        self,
        feature_table: str,
        project: str,
        ingestion_timespan: int,
        cron_schedule: str,
    ):
        """
        Launch Scheduled Ingestion Job from Batch Source to Online Store for given feature table

        Args:
            feature_table:  FeatureTable that will be ingested into the online store
            project: Project name
            ingestion_timespan: Days of data which will be ingestion per job. The boundaries
                on which to filter the source are [end of day of execution date - ingestion_timespan (days) ,
                end of day of execution date)
            cron_schedule: Cron schedule expression

        """
        if not croniter.is_valid(cron_schedule):
            raise RuntimeError(f"{cron_schedule} is not a valid cron expression")
        request = ScheduleOfflineToOnlineIngestionJobRequest(
            project=project,
            table_name=feature_table,
            ingestion_timespan=ingestion_timespan,
            cron_schedule=cron_schedule,
        )
        self._job_service.ScheduleOfflineToOnlineIngestionJob(request)

    def get_historical_features(
        self,
        feature_refs: List[str],
        entity_source: Union[FileSource, BigQuerySource],
        output_location: str,
        project: str,
        output_format: str = "parquet",
    ) -> GetHistoricalFeaturesResponse:
        """
        Launch a historical feature retrieval job.

        Args:
            feature_refs: List of feature references that will be returned for each entity.
                Each feature reference should have the following format:
                "feature_table:feature" where "feature_table" & "feature" refer to
                the feature and feature table names respectively.
            entity_source (Union[FileSource, BigQuerySource]): Source for the entity rows.
                It is assumed that the column event_timestamp is present
                in the dataframe, and is of type datetime without timezone information.
            output_location: Specifies the directory in a GCS bucket to write the exported feature data files
            project: Feast project name
            output_format: Spark output format

        Returns:
                Returns a retrieval job object that can be used to monitor retrieval
                progress asynchronously, and can be used to materialize the
                results.
        """

        return self._job_service.GetHistoricalFeatures(
            GetHistoricalFeaturesRequest(
                feature_refs=feature_refs,
                entity_source=entity_source.to_proto(),
                project=project,
                output_format=output_format,
                output_location=output_location,
            ),
        )

    def get_job(self, job_id: str) -> Job:
        """
        Get job details
        Args:
            job_id: spark job id

        Returns: Job protobuf object
        """
        request = GetJobRequest(job_id=job_id)
        response = self._job_service.GetJob(request)
        return response.job

    def list_job(self, table: str = "", project: str = "", include_terminated=True, job_type: JobType = INVALID_JOB) -> List[Job]:
        """
        List job details
        Args:
            include_terminated: include terminated jobs
            table: feature table name
            project: feast project name
            job_type: job type

        Returns: List of Job protobuf object
        """
        request = ListJobsRequest(table_name=table, project=project, include_terminated=include_terminated, type=job_type)
        response = self._job_service.ListJobs(request)
        return response.jobs

    def list_scheduled_job(self, project = "", table_name = "") -> List[ScheduledJob]:
        """
        List scheduled jobs
        Args:
            project: Filter by project, if non empty.
            table_name: Filter by table name, if non empty
        Returns: List of Scheduled Job protobuf object
        """
        request = ListScheduledJobsRequest(project=project, table_name=table_name)
        response = self._job_service.ListScheduledJobs(request)
        return response.jobs

    def delete_feature_table(self, feature_table: str, project: str):
        """
        Delete Feature Table
        Args:
            feature_table: feature table name
            project: project name
        """
        request = DeleteFeatureTableRequest(name=feature_table, project=project)
        self._core_service.DeleteFeatureTable(request)

    def cancel_job(self, job_id: str):
        """
        Cancel job
        Args:
            job_id: spark job id
        """
        request = CancelJobRequest(job_id=job_id)
        self._job_service.CancelJob(request)

    def unschedule_job(self, job_id: str):
        """
        Unschedule job
        Args:
            job_id: spark job id
        """
        request = UnscheduleJobRequest(job_id=job_id)
        self._job_service.UnscheduleJob(request)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class FileFormat(google.protobuf.message.Message):
    """Defines the file format encoding the features/entity data in files"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class ParquetFormat(google.protobuf.message.Message):
        """Defines options for the Parquet data format"""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        def __init__(
            self,
        ) -> None: ...

    PARQUET_FORMAT_FIELD_NUMBER: builtins.int
    @property
    def parquet_format(self) -> global___FileFormat.ParquetFormat: ...
    def __init__(
        self,
        *,
        parquet_format: global___FileFormat.ParquetFormat | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["format", b"format", "parquet_format", b"parquet_format"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["format", b"format", "parquet_format", b"parquet_format"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["format", b"format"]) -> typing_extensions.Literal["parquet_format"] | None: ...

global___FileFormat = FileFormat

@typing_extensions.final
class StreamFormat(google.protobuf.message.Message):
    """Defines the data format encoding features/entity data in data streams"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class ProtoFormat(google.protobuf.message.Message):
        """Defines options for the protobuf data format"""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        CLASS_PATH_FIELD_NUMBER: builtins.int
        class_path: builtins.str
        """Classpath to the generated Java Protobuf class that can be used to decode
        Feature data from the obtained stream message
        """
        def __init__(
            self,
            *,
            class_path: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["class_path", b"class_path"]) -> None: ...

    @typing_extensions.final
    class AvroFormat(google.protobuf.message.Message):
        """Defines options for the avro data format"""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        SCHEMA_JSON_FIELD_NUMBER: builtins.int
        schema_json: builtins.str
        """Optional if used in a File DataSource as schema is embedded in avro file.
        Specifies the schema of the Avro message as JSON string.
        """
        def __init__(
            self,
            *,
            schema_json: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["schema_json", b"schema_json"]) -> None: ...

    AVRO_FORMAT_FIELD_NUMBER: builtins.int
    PROTO_FORMAT_FIELD_NUMBER: builtins.int
    @property
    def avro_format(self) -> global___StreamFormat.AvroFormat: ...
    @property
    def proto_format(self) -> global___StreamFormat.ProtoFormat: ...
    def __init__(
        self,
        *,
        avro_format: global___StreamFormat.AvroFormat | None = ...,
        proto_format: global___StreamFormat.ProtoFormat | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["avro_format", b"avro_format", "format", b"format", "proto_format", b"proto_format"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["avro_format", b"avro_format", "format", b"format", "proto_format", b"proto_format"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["format", b"format"]) -> typing_extensions.Literal["avro_format", "proto_format"] | None: ...

global___StreamFormat = StreamFormat



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/serving/ServingService.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from feast.types import Value_pb2 as feast_dot_types_dot_Value__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"feast/serving/ServingService.proto\x12\rfeast.serving\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x17\x66\x65\x61st/types/Value.proto\"\x1c\n\x1aGetFeastServingInfoRequest\".\n\x1bGetFeastServingInfoResponse\x12\x0f\n\x07version\x18\x01 \x01(\t\"7\n\x10\x46\x65\x61tureReference\x12\x15\n\rfeature_table\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\"\xf5\x02\n\x18GetOnlineFeaturesRequest\x12\x31\n\x08\x66\x65\x61tures\x18\x04 \x03(\x0b\x32\x1f.feast.serving.FeatureReference\x12\x46\n\x0b\x65ntity_rows\x18\x02 \x03(\x0b\x32\x31.feast.serving.GetOnlineFeaturesRequest.EntityRow\x12\x0f\n\x07project\x18\x05 \x01(\t\x1a\xcc\x01\n\tEntityRow\x12-\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12M\n\x06\x66ields\x18\x02 \x03(\x0b\x32=.feast.serving.GetOnlineFeaturesRequest.EntityRow.FieldsEntry\x1a\x41\n\x0b\x46ieldsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12!\n\x05value\x18\x02 \x01(\x0b\x32\x12.feast.types.Value:\x02\x38\x01\"\xb7\x03\n\x1bGetOnlineFeaturesResponseV2\x12L\n\x0c\x66ield_values\x18\x01 \x03(\x0b\x32\x36.feast.serving.GetOnlineFeaturesResponseV2.FieldValues\x1a\xc9\x02\n\x0b\x46ieldValues\x12R\n\x06\x66ields\x18\x01 \x03(\x0b\x32\x42.feast.serving.GetOnlineFeaturesResponseV2.FieldValues.FieldsEntry\x12V\n\x08statuses\x18\x02 \x03(\x0b\x32\x44.feast.serving.GetOnlineFeaturesResponseV2.FieldValues.StatusesEntry\x1a\x41\n\x0b\x46ieldsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12!\n\x05value\x18\x02 \x01(\x0b\x32\x12.feast.types.Value:\x02\x38\x01\x1aK\n\rStatusesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12)\n\x05value\x18\x02 \x01(\x0e\x32\x1a.feast.serving.FieldStatus:\x02\x38\x01\"\x87\x02\n\x19GetOnlineFeaturesResponse\x12\x42\n\x08metadata\x18\x01 \x01(\x0b\x32\x30.feast.serving.GetOnlineFeaturesResponseMetadata\x12\x45\n\x07results\x18\x02 \x03(\x0b\x32\x34.feast.serving.GetOnlineFeaturesResponse.FieldVector\x1a_\n\x0b\x46ieldVector\x12\"\n\x06values\x18\x01 \x03(\x0b\x32\x12.feast.types.Value\x12,\n\x08statuses\x18\x02 \x03(\x0e\x32\x1a.feast.serving.FieldStatus\"R\n!GetOnlineFeaturesResponseMetadata\x12-\n\x0b\x66ield_names\x18\x01 \x01(\x0b\x32\x18.feast.serving.FieldList\"\x18\n\tFieldList\x12\x0b\n\x03val\x18\x01 \x03(\t*r\n\x0b\x46ieldStatus\x12\x0b\n\x07INVALID\x10\x00\x12\x0b\n\x07PRESENT\x10\x01\x12\x0e\n\nNULL_VALUE\x10\x02\x12\r\n\tNOT_FOUND\x10\x03\x12\x13\n\x0fOUTSIDE_MAX_AGE\x10\x04\x12\x15\n\x11INGESTION_FAILURE\x10\x05\x32\xd2\x02\n\x0eServingService\x12l\n\x13GetFeastServingInfo\x12).feast.serving.GetFeastServingInfoRequest\x1a*.feast.serving.GetFeastServingInfoResponse\x12j\n\x13GetOnlineFeaturesV2\x12\'.feast.serving.GetOnlineFeaturesRequest\x1a*.feast.serving.GetOnlineFeaturesResponseV2\x12\x66\n\x11GetOnlineFeatures\x12\'.feast.serving.GetOnlineFeaturesRequest\x1a(.feast.serving.GetOnlineFeaturesResponseB\x85\x01\n!dev.caraml.store.protobuf.servingB\x13ServingServiceProtoZKgithub.com/caraml-dev/caraml-store/caraml-store-sdk/go/protos/feast/servingb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'feast.serving.ServingService_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n!dev.caraml.store.protobuf.servingB\023ServingServiceProtoZKgithub.com/caraml-dev/caraml-store/caraml-store-sdk/go/protos/feast/serving'
  _GETONLINEFEATURESREQUEST_ENTITYROW_FIELDSENTRY._options = None
  _GETONLINEFEATURESREQUEST_ENTITYROW_FIELDSENTRY._serialized_options = b'8\001'
  _GETONLINEFEATURESRESPONSEV2_FIELDVALUES_FIELDSENTRY._options = None
  _GETONLINEFEATURESRESPONSEV2_FIELDVALUES_FIELDSENTRY._serialized_options = b'8\001'
  _GETONLINEFEATURESRESPONSEV2_FIELDVALUES_STATUSESENTRY._options = None
  _GETONLINEFEATURESRESPONSEV2_FIELDVALUES_STATUSESENTRY._serialized_options = b'8\001'
  _FIELDSTATUS._serialized_start=1440
  _FIELDSTATUS._serialized_end=1554
  _GETFEASTSERVINGINFOREQUEST._serialized_start=111
  _GETFEASTSERVINGINFOREQUEST._serialized_end=139
  _GETFEASTSERVINGINFORESPONSE._serialized_start=141
  _GETFEASTSERVINGINFORESPONSE._serialized_end=187
  _FEATUREREFERENCE._serialized_start=189
  _FEATUREREFERENCE._serialized_end=244
  _GETONLINEFEATURESREQUEST._serialized_start=247
  _GETONLINEFEATURESREQUEST._serialized_end=620
  _GETONLINEFEATURESREQUEST_ENTITYROW._serialized_start=416
  _GETONLINEFEATURESREQUEST_ENTITYROW._serialized_end=620
  _GETONLINEFEATURESREQUEST_ENTITYROW_FIELDSENTRY._serialized_start=555
  _GETONLINEFEATURESREQUEST_ENTITYROW_FIELDSENTRY._serialized_end=620
  _GETONLINEFEATURESRESPONSEV2._serialized_start=623
  _GETONLINEFEATURESRESPONSEV2._serialized_end=1062
  _GETONLINEFEATURESRESPONSEV2_FIELDVALUES._serialized_start=733
  _GETONLINEFEATURESRESPONSEV2_FIELDVALUES._serialized_end=1062
  _GETONLINEFEATURESRESPONSEV2_FIELDVALUES_FIELDSENTRY._serialized_start=555
  _GETONLINEFEATURESRESPONSEV2_FIELDVALUES_FIELDSENTRY._serialized_end=620
  _GETONLINEFEATURESRESPONSEV2_FIELDVALUES_STATUSESENTRY._serialized_start=987
  _GETONLINEFEATURESRESPONSEV2_FIELDVALUES_STATUSESENTRY._serialized_end=1062
  _GETONLINEFEATURESRESPONSE._serialized_start=1065
  _GETONLINEFEATURESRESPONSE._serialized_end=1328
  _GETONLINEFEATURESRESPONSE_FIELDVECTOR._serialized_start=1233
  _GETONLINEFEATURESRESPONSE_FIELDVECTOR._serialized_end=1328
  _GETONLINEFEATURESRESPONSEMETADATA._serialized_start=1330
  _GETONLINEFEATURESRESPONSEMETADATA._serialized_end=1412
  _FIELDLIST._serialized_start=1414
  _FIELDLIST._serialized_end=1438
  _SERVINGSERVICE._serialized_start=1557
  _SERVINGSERVICE._serialized_end=1895
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/core/FeatureTable.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from feast.core import DataSource_pb2 as feast_dot_core_dot_DataSource__pb2
from feast.core import Feature_pb2 as feast_dot_core_dot_Feature__pb2
from feast.core import OnlineStore_pb2 as feast_dot_core_dot_OnlineStore__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1d\x66\x65\x61st/core/FeatureTable.proto\x12\nfeast.core\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1b\x66\x65\x61st/core/DataSource.proto\x1a\x18\x66\x65\x61st/core/Feature.proto\x1a\x1c\x66\x65\x61st/core/OnlineStore.proto\"f\n\x0c\x46\x65\x61tureTable\x12*\n\x04spec\x18\x01 \x01(\x0b\x32\x1c.feast.core.FeatureTableSpec\x12*\n\x04meta\x18\x02 \x01(\x0b\x32\x1c.feast.core.FeatureTableMeta\"\xb6\x03\n\x10\x46\x65\x61tureTableSpec\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08\x65ntities\x18\x03 \x03(\t\x12)\n\x08\x66\x65\x61tures\x18\x04 \x03(\x0b\x32\x17.feast.core.FeatureSpec\x12\x38\n\x06labels\x18\x05 \x03(\x0b\x32(.feast.core.FeatureTableSpec.LabelsEntry\x12*\n\x07max_age\x18\x06 \x01(\x0b\x32\x19.google.protobuf.Duration\x12,\n\x0c\x62\x61tch_source\x18\x07 \x01(\x0b\x32\x16.feast.core.DataSource\x12-\n\rstream_source\x18\x08 \x01(\x0b\x32\x16.feast.core.DataSource\x12\x36\n\x13staleness_threshold\x18\t \x01(\x0b\x32\x19.google.protobuf.Duration\x12-\n\x0conline_store\x18\n \x01(\x0b\x32\x17.feast.core.OnlineStore\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xa5\x01\n\x10\x46\x65\x61tureTableMeta\x12\x35\n\x11\x63reated_timestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12:\n\x16last_updated_timestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x10\n\x08revision\x18\x03 \x01(\x03\x12\x0c\n\x04hash\x18\x04 \x01(\tB}\n\x1e\x64\x65v.caraml.store.protobuf.coreB\x11\x46\x65\x61tureTableProtoZHgithub.com/caraml-dev/caraml-store/caraml-store-sdk/go/protos/feast/coreb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'feast.core.FeatureTable_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\036dev.caraml.store.protobuf.coreB\021FeatureTableProtoZHgithub.com/caraml-dev/caraml-store/caraml-store-sdk/go/protos/feast/core'
  _FEATURETABLESPEC_LABELSENTRY._options = None
  _FEATURETABLESPEC_LABELSENTRY._serialized_options = b'8\001'
  _FEATURETABLE._serialized_start=195
  _FEATURETABLE._serialized_end=297
  _FEATURETABLESPEC._serialized_start=300
  _FEATURETABLESPEC._serialized_end=738
  _FEATURETABLESPEC_LABELSENTRY._serialized_start=693
  _FEATURETABLESPEC_LABELSENTRY._serialized_end=738
  _FEATURETABLEMETA._serialized_start=741
  _FEATURETABLEMETA._serialized_end=906
# @@protoc_insertion_point(module_scope)



from pathlib import Path

from google.protobuf.timestamp_pb2 import Timestamp

from feast.core.DataFormat_pb2 import StreamFormat as StreamFormatProto
from feast.core.DataSource_pb2 import DataSource as DataSourceProto
from feast.core.OnlineStore_pb2 import OnlineStore as OnlineStoreProto, StoreType as StoreTypeProto
from feast.core.SparkOverride_pb2 import SparkOverride as SparkOverrideProto
from feast.feature_table import FeatureTable
from feast.core.FeatureTable_pb2 import FeatureTable as FeatureTableProto, FeatureTableSpec as FeatureTableSpecProto, \
    FeatureTableMeta as FeatureTableMetaProto
from feast.core.Feature_pb2 import FeatureSpec as FeatureSpecProto
from feast.types.Value_pb2 import ValueType


def test_feature_table_conversion():
    test_spec_file_path = Path(__file__).resolve().parent / "data" / "test_feature_table.yaml"
    feature_table = FeatureTable.from_yaml(str(test_spec_file_path))
    expected_feature_table_proto = FeatureTableProto(
        spec=FeatureTableSpecProto(
            name="transaction",
            entities=["merchant"],
            features=[
                FeatureSpecProto(name="avg_transaction", value_type=ValueType.DOUBLE)
            ],
            batch_source=DataSourceProto(
                type=DataSourceProto.BATCH_BIGQUERY,
                event_timestamp_column="datetime",
                bigquery_options=DataSourceProto.BigQueryOptions(
                    table_ref="project:dataset.table",
                    spark_override=SparkOverrideProto()
                ),
            ),
            stream_source=DataSourceProto(
                type=DataSourceProto.STREAM_KAFKA,
                event_timestamp_column="event_timestamp",
                kafka_options=DataSourceProto.KafkaOptions(
                    bootstrap_servers="localhost:6668",
                    topic="transaction_topic",
                    message_format=StreamFormatProto(
                        proto_format=StreamFormatProto.ProtoFormat(
                            class_path="com.transaction.Message"
                        )
                    ),
                    spark_override=SparkOverrideProto()
                )
            ),
            online_store=OnlineStoreProto(
                name="bigtable",
                type=StoreTypeProto.BIGTABLE
            )
        ),
        meta=FeatureTableMetaProto(
            created_timestamp=Timestamp()
        )

    )
    assert feature_table.to_proto() == expected_feature_table_proto
    assert feature_table == FeatureTable.from_proto(expected_feature_table_proto)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import feast.core.DataFormat_pb2
import feast.core.SparkOverride_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class DataSource(google.protobuf.message.Message):
    """Defines a Data Source that can be used source Feature data"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _SourceType:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _SourceTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[DataSource._SourceType.ValueType], builtins.type):  # noqa: F821
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        INVALID: DataSource._SourceType.ValueType  # 0
        BATCH_FILE: DataSource._SourceType.ValueType  # 1
        BATCH_BIGQUERY: DataSource._SourceType.ValueType  # 2
        STREAM_KAFKA: DataSource._SourceType.ValueType  # 3

    class SourceType(_SourceType, metaclass=_SourceTypeEnumTypeWrapper):
        """Type of Data Source."""

    INVALID: DataSource.SourceType.ValueType  # 0
    BATCH_FILE: DataSource.SourceType.ValueType  # 1
    BATCH_BIGQUERY: DataSource.SourceType.ValueType  # 2
    STREAM_KAFKA: DataSource.SourceType.ValueType  # 3

    @typing_extensions.final
    class FieldMappingEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    @typing_extensions.final
    class FileOptions(google.protobuf.message.Message):
        """Defines options for DataSource that sources features from a file"""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        FILE_FORMAT_FIELD_NUMBER: builtins.int
        FILE_URL_FIELD_NUMBER: builtins.int
        SPARK_OVERRIDE_FIELD_NUMBER: builtins.int
        @property
        def file_format(self) -> feast.core.DataFormat_pb2.FileFormat: ...
        file_url: builtins.str
        """Target URL of file to retrieve and source features from.
        s3://path/to/file for AWS S3 storage
        gs://path/to/file for GCP GCS storage
        file:///path/to/file for local storage
        """
        @property
        def spark_override(self) -> feast.core.SparkOverride_pb2.SparkOverride:
            """Allow users to override some configuration for the ingestion jobs."""
        def __init__(
            self,
            *,
            file_format: feast.core.DataFormat_pb2.FileFormat | None = ...,
            file_url: builtins.str = ...,
            spark_override: feast.core.SparkOverride_pb2.SparkOverride | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["file_format", b"file_format", "spark_override", b"spark_override"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["file_format", b"file_format", "file_url", b"file_url", "spark_override", b"spark_override"]) -> None: ...

    @typing_extensions.final
    class BigQueryOptions(google.protobuf.message.Message):
        """Defines options for DataSource that sources features from a BigQuery Query"""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TABLE_REF_FIELD_NUMBER: builtins.int
        SPARK_OVERRIDE_FIELD_NUMBER: builtins.int
        table_ref: builtins.str
        """Full table reference in the form of [project:dataset.table]"""
        @property
        def spark_override(self) -> feast.core.SparkOverride_pb2.SparkOverride:
            """Allow users to override some configuration for the ingestion jobs."""
        def __init__(
            self,
            *,
            table_ref: builtins.str = ...,
            spark_override: feast.core.SparkOverride_pb2.SparkOverride | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["spark_override", b"spark_override"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["spark_override", b"spark_override", "table_ref", b"table_ref"]) -> None: ...

    @typing_extensions.final
    class KafkaOptions(google.protobuf.message.Message):
        """Defines options for DataSource that sources features from Kafka messages.
        Each message should be a Protobuf that can be decoded with the generated
        Java Protobuf class at the given class path
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        BOOTSTRAP_SERVERS_FIELD_NUMBER: builtins.int
        TOPIC_FIELD_NUMBER: builtins.int
        MESSAGE_FORMAT_FIELD_NUMBER: builtins.int
        SPARK_OVERRIDE_FIELD_NUMBER: builtins.int
        bootstrap_servers: builtins.str
        """Comma separated list of Kafka bootstrap servers. Used for feature tables without a defined source host[:port]]"""
        topic: builtins.str
        """Kafka topic to collect feature data from."""
        @property
        def message_format(self) -> feast.core.DataFormat_pb2.StreamFormat:
            """Defines the stream data format encoding feature/entity data in Kafka messages."""
        @property
        def spark_override(self) -> feast.core.SparkOverride_pb2.SparkOverride:
            """Allow users to override some configuration for the ingestion jobs."""
        def __init__(
            self,
            *,
            bootstrap_servers: builtins.str = ...,
            topic: builtins.str = ...,
            message_format: feast.core.DataFormat_pb2.StreamFormat | None = ...,
            spark_override: feast.core.SparkOverride_pb2.SparkOverride | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["message_format", b"message_format", "spark_override", b"spark_override"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["bootstrap_servers", b"bootstrap_servers", "message_format", b"message_format", "spark_override", b"spark_override", "topic", b"topic"]) -> None: ...

    TYPE_FIELD_NUMBER: builtins.int
    FIELD_MAPPING_FIELD_NUMBER: builtins.int
    EVENT_TIMESTAMP_COLUMN_FIELD_NUMBER: builtins.int
    DATE_PARTITION_COLUMN_FIELD_NUMBER: builtins.int
    CREATED_TIMESTAMP_COLUMN_FIELD_NUMBER: builtins.int
    FILE_OPTIONS_FIELD_NUMBER: builtins.int
    BIGQUERY_OPTIONS_FIELD_NUMBER: builtins.int
    KAFKA_OPTIONS_FIELD_NUMBER: builtins.int
    type: global___DataSource.SourceType.ValueType
    @property
    def field_mapping(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """Defines mapping between fields in the sourced data 
        and fields in parent FeatureTable.
        """
    event_timestamp_column: builtins.str
    """Must specify event timestamp column name"""
    date_partition_column: builtins.str
    """(Optional) Specify partition column
    useful for file sources
    """
    created_timestamp_column: builtins.str
    """Must specify creation timestamp column name"""
    @property
    def file_options(self) -> global___DataSource.FileOptions: ...
    @property
    def bigquery_options(self) -> global___DataSource.BigQueryOptions: ...
    @property
    def kafka_options(self) -> global___DataSource.KafkaOptions: ...
    def __init__(
        self,
        *,
        type: global___DataSource.SourceType.ValueType = ...,
        field_mapping: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        event_timestamp_column: builtins.str = ...,
        date_partition_column: builtins.str = ...,
        created_timestamp_column: builtins.str = ...,
        file_options: global___DataSource.FileOptions | None = ...,
        bigquery_options: global___DataSource.BigQueryOptions | None = ...,
        kafka_options: global___DataSource.KafkaOptions | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["bigquery_options", b"bigquery_options", "file_options", b"file_options", "kafka_options", b"kafka_options", "options", b"options"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["bigquery_options", b"bigquery_options", "created_timestamp_column", b"created_timestamp_column", "date_partition_column", b"date_partition_column", "event_timestamp_column", b"event_timestamp_column", "field_mapping", b"field_mapping", "file_options", b"file_options", "kafka_options", b"kafka_options", "options", b"options", "type", b"type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["options", b"options"]) -> typing_extensions.Literal["file_options", "bigquery_options", "kafka_options"] | None: ...

global___DataSource = DataSource



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/core/OnlineStore.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1c\x66\x65\x61st/core/OnlineStore.proto\x12\nfeast.core\"U\n\x0bOnlineStore\x12\x0c\n\x04name\x18\x01 \x01(\t\x12#\n\x04type\x18\x02 \x01(\x0e\x32\x15.feast.core.StoreType\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t*:\n\tStoreType\x12\t\n\x05UNSET\x10\x00\x12\x0c\n\x08\x42IGTABLE\x10\x01\x12\t\n\x05REDIS\x10\x02\x12\t\n\x05HBASE\x10\x03\x42|\n\x1e\x64\x65v.caraml.store.protobuf.coreB\x10OnlineStoreProtoZHgithub.com/caraml-dev/caraml-store/caraml-store-sdk/go/protos/feast/coreb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'feast.core.OnlineStore_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\036dev.caraml.store.protobuf.coreB\020OnlineStoreProtoZHgithub.com/caraml-dev/caraml-store/caraml-store-sdk/go/protos/feast/core'
  _STORETYPE._serialized_start=131
  _STORETYPE._serialized_end=189
  _ONLINESTORE._serialized_start=44
  _ONLINESTORE._serialized_end=129
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/core/LegacyJobService.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from feast_spark.api import JobService_pb2 as feast__spark_dot_api_dot_JobService__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!feast/core/LegacyJobService.proto\x12\nfeast.core\x1a feast_spark/api/JobService.proto2\xe9\x02\n\nJobService\x12\x97\x01\n StartOfflineToOnlineIngestionJob\x12\x38.feast_spark.api.StartOfflineToOnlineIngestionJobRequest\x1a\x39.feast_spark.api.StartOfflineToOnlineIngestionJobResponse\x12v\n\x15GetHistoricalFeatures\x12-.feast_spark.api.GetHistoricalFeaturesRequest\x1a..feast_spark.api.GetHistoricalFeaturesResponse\x12I\n\x06GetJob\x12\x1e.feast_spark.api.GetJobRequest\x1a\x1f.feast_spark.api.GetJobResponseB}\n dev.caraml.store.protobuf.compatB\x0fJobServiceProtoZHgithub.com/caraml-dev/caraml-store/caraml-store-sdk/go/protos/feast/coreb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'feast.core.LegacyJobService_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n dev.caraml.store.protobuf.compatB\017JobServiceProtoZHgithub.com/caraml-dev/caraml-store/caraml-store-sdk/go/protos/feast/core'
  _JOBSERVICE._serialized_start=84
  _JOBSERVICE._serialized_end=445
# @@protoc_insertion_point(module_scope)



from typing import Dict, List, MutableMapping, Optional, Union

import yaml
from google.protobuf import json_format
from google.protobuf.internal.well_known_types import Duration, Timestamp
from google.protobuf.json_format import MessageToDict, MessageToJson

from feast.core.FeatureTable_pb2 import FeatureTable as FeatureTableProto
from feast.core.FeatureTable_pb2 import FeatureTableMeta as FeatureTableMetaProto
from feast.core.FeatureTable_pb2 import FeatureTableSpec as FeatureTableSpecProto
from feast.data_source import (
    BigQuerySource,
    FileSource,
    KafkaSource, new_batch_source_from_proto, new_stream_source_from_proto,
)
from feast.feature import Feature
from feast.loaders.file import yaml_loader
from feast.online_store import OnlineStore
from feast.value_type import ValueType


class FeatureTable:
    """
    Represents a collection of features and associated metadata.
    """

    def __init__(
        self,
        name: str,
        entities: List[str],
        features: List[Feature],
        batch_source: Optional[Union[BigQuerySource, FileSource]] = None,
        stream_source: Optional[KafkaSource] = None,
        max_age: Optional[Duration] = None,
        labels: Optional[MutableMapping[str, str]] = None,
        online_store: Optional[OnlineStore] = None,
    ):
        self._name = name
        self._entities = entities
        self._features = features
        self._batch_source = batch_source
        self._stream_source = stream_source
        if labels is None:
            self._labels = dict()  # type: MutableMapping[str, str]
        else:
            self._labels = labels

        self._max_age = max_age
        self._created_timestamp: Optional[Timestamp] = None
        self._last_updated_timestamp: Optional[Timestamp] = None
        self._online_store = online_store

    def __str__(self):
        return str(MessageToJson(self.to_proto()))

    def __eq__(self, other):
        if not isinstance(other, FeatureTable):
            raise TypeError(
                "Comparisons should only involve FeatureTable class objects."
            )

        if (
            self.labels != other.labels
            or self.name != other.name
            or self.max_age != other.max_age
        ):
            return False

        if sorted(self.entities) != sorted(other.entities):
            return False
        if sorted(self.features) != sorted(other.features):
            return False
        if self.batch_source != other.batch_source:
            return False
        if self.stream_source != other.stream_source:
            return False
        if self._online_store != other._online_store:
            return False

        return True

    @property
    def name(self):
        """
        Returns the name of this feature table
        """
        return self._name

    @name.setter
    def name(self, name: str):
        """
        Sets the name of this feature table
        """
        self._name = name

    @property
    def entities(self):
        """
        Returns the entities of this feature table
        """
        return self._entities

    @entities.setter
    def entities(self, entities: List[str]):
        """
        Sets the entities of this feature table
        """
        self._entities = entities

    @property
    def features(self):
        """
        Returns the features of this feature table
        """
        return self._features

    @features.setter
    def features(self, features: List[Feature]):
        """
        Sets the features of this feature table
        """
        self._features = features

    @property
    def batch_source(self):
        """
        Returns the batch source of this feature table
        """
        return self._batch_source

    @batch_source.setter
    def batch_source(self, batch_source: Optional[Union[BigQuerySource, FileSource]]):
        """
        Sets the batch source of this feature table
        """
        self._batch_source = batch_source

    @property
    def stream_source(self):
        """
        Returns the stream source of this feature table
        """
        return self._stream_source

    @stream_source.setter
    def stream_source(self, stream_source: KafkaSource):
        """
        Sets the stream source of this feature table
        """
        self._stream_source = stream_source

    @property
    def max_age(self):
        """
        Returns the maximum age of this feature table. This is the total maximum
        amount of staleness that will be allowed during feature retrieval for
        each specific feature that is looked up.
        """
        return self._max_age

    @max_age.setter
    def max_age(self, max_age: Duration):
        """
        Set the maximum age for this feature table
        """
        self._max_age = max_age

    @property
    def labels(self):
        """
        Returns the labels of this feature table. This is the user defined metadata
        defined as a dictionary.
        """
        return self._labels

    @labels.setter
    def labels(self, labels: MutableMapping[str, str]):
        """
        Set the labels for this feature table
        """
        self._labels = labels

    @property
    def created_timestamp(self):
        """
        Returns the created_timestamp of this feature table
        """
        return self._created_timestamp

    @property
    def last_updated_timestamp(self):
        """
        Returns the last_updated_timestamp of this feature table
        """
        return self._last_updated_timestamp

    @property
    def online_store(self):
        """
        Returns the online store of this feature table
        """
        return self._online_store

    @online_store.setter
    def online_store(self, online_store: OnlineStore):
        """
        Sets the stream source of this feature table
        """
        self._online_store = online_store

    def add_feature(self, feature: Feature):
        """
        Adds a new feature to the feature table.
        """
        self.features.append(feature)

    def is_valid(self):
        """
        Validates the state of a feature table locally. Raises an exception
        if feature table is invalid.
        """

        if not self.name:
            raise ValueError("No name found in feature table.")

        if not self.entities:
            raise ValueError("No entities found in feature table {self.name}.")

    @classmethod
    def from_yaml(cls, yml: str):
        """
        Creates a feature table from a YAML string body or a file path
        Args:
            yml: Either a file path containing a yaml file or a YAML string
        Returns:
            Returns a FeatureTable object based on the YAML file
        """

        return cls.from_dict(yaml_loader(yml, load_single=True))

    @classmethod
    def from_dict(cls, ft_dict):
        """
        Creates a feature table from a dict
        Args:
            ft_dict: A dict representation of a feature table
        Returns:
            Returns a FeatureTable object based on the feature table dict
        """

        feature_table_proto = json_format.ParseDict(
            ft_dict, FeatureTableProto(), ignore_unknown_fields=True
        )

        return cls.from_proto(feature_table_proto)

    @classmethod
    def from_proto(cls, feature_table_proto: FeatureTableProto):
        """
        Creates a feature table from a protobuf representation of a feature table
        Args:
            feature_table_proto: A protobuf representation of a feature table
        Returns:
            Returns a FeatureTableProto object based on the feature table protobuf
        """

        feature_table = cls(
            name=feature_table_proto.spec.name,
            entities=[entity for entity in feature_table_proto.spec.entities],
            features=[
                Feature(
                    name=feature.name,
                    dtype=ValueType(feature.value_type),
                    labels=feature.labels,
                )
                for feature in feature_table_proto.spec.features
            ],
            labels=feature_table_proto.spec.labels,
            max_age=(
                None
                if feature_table_proto.spec.max_age.seconds == 0
                and feature_table_proto.spec.max_age.nanos == 0
                else feature_table_proto.spec.max_age
            ),
            batch_source=None if not feature_table_proto.spec.batch_source.ByteSize() else new_batch_source_from_proto(feature_table_proto.spec.batch_source),
            stream_source=(
                None
                if not feature_table_proto.spec.stream_source.ByteSize()
                else new_stream_source_from_proto(feature_table_proto.spec.stream_source)
            ),
            online_store=OnlineStore.from_proto(feature_table_proto.spec.online_store),
        )

        feature_table._created_timestamp = feature_table_proto.meta.created_timestamp

        return feature_table

    def to_proto(self) -> FeatureTableProto:
        """
        Converts a feature table object to its protobuf representation
        Returns:
            FeatureTableProto protobuf
        """

        meta = FeatureTableMetaProto(
            created_timestamp=self.created_timestamp,
            last_updated_timestamp=self.last_updated_timestamp,
        )

        spec = FeatureTableSpecProto(
            name=self.name,
            entities=self.entities,
            features=[
                feature.to_proto() if type(feature) == Feature else feature
                for feature in self.features
            ],
            labels=self.labels,
            max_age=None if self.max_age else self.max_age,
            batch_source=self.batch_source.to_proto(),
            stream_source=None if self.stream_source is None else self.stream_source.to_proto(),
            online_store=self.online_store.to_proto()
            if self.online_store is not None
            else None,
        )

        return FeatureTableProto(spec=spec, meta=meta)

    def to_spec_proto(self) -> FeatureTableSpecProto:
        """
        Converts an FeatureTableProto object to its protobuf representation.
        Used when passing FeatureTableSpecProto object to Feast request.
        Returns:
            FeatureTableSpecProto protobuf
        """

        spec = FeatureTableSpecProto(
            name=self.name,
            entities=self.entities,
            features=[
                feature.to_proto() if type(feature) == Feature else feature
                for feature in self.features
            ],
            labels=self.labels,
            max_age=self.max_age,
            batch_source=(
                self.batch_source.to_proto()
                if self.batch_source is not None
                else None
            ),
            stream_source=(
                self.stream_source.to_proto()
                if self.stream_source is not None
                else None
            ),
            online_store=self.online_store.to_proto()
            if self.online_store is not None
            else None,
        )

        return spec

    def to_dict(self) -> Dict:
        """
        Converts feature table to dict
        :return: Dictionary object representation of feature table
        """
        feature_table_dict = MessageToDict(self.to_proto())

        # Remove meta when empty for more readable exports
        if feature_table_dict["meta"] == {}:
            del feature_table_dict["meta"]

        return feature_table_dict

    def to_yaml(self):
        """
        Converts a feature table to a YAML string.
        :return: Feature table string returned in YAML format
        """
        feature_table_dict = self.to_dict()
        return yaml.dump(feature_table_dict, allow_unicode=True, sort_keys=False)

    def _update_from_feature_table(self, feature_table):
        """
        Deep replaces one feature table with another
        Args:
            feature_table: Feature table to use as a source of configuration
        """

        self.name = feature_table.name
        self.entities = feature_table.entities
        self.features = feature_table.features
        self.labels = feature_table.labels
        self.max_age = feature_table.max_age
        self.batch_source = feature_table.batch_source
        self.stream_source = feature_table.stream_source
        self.online_store = feature_table.online_store
        self._created_timestamp = feature_table.created_timestamp
        self._last_updated_timestamp = feature_table.last_updated_timestamp

    def __repr__(self):
        return f"FeatureTable <{self.name}>"



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _StoreType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _StoreTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_StoreType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    UNSET: _StoreType.ValueType  # 0
    """Unset"""
    BIGTABLE: _StoreType.ValueType  # 1
    """google cloud NOSQL database service"""
    REDIS: _StoreType.ValueType  # 2
    """redis in-memory database"""
    HBASE: _StoreType.ValueType  # 3
    """HBase database"""

class StoreType(_StoreType, metaclass=_StoreTypeEnumTypeWrapper): ...

UNSET: StoreType.ValueType  # 0
"""Unset"""
BIGTABLE: StoreType.ValueType  # 1
"""google cloud NOSQL database service"""
REDIS: StoreType.ValueType  # 2
"""redis in-memory database"""
HBASE: StoreType.ValueType  # 3
"""HBase database"""
global___StoreType = StoreType

@typing_extensions.final
class OnlineStore(google.protobuf.message.Message):
    """OnlineStore provides a location where Feast reads and writes feature values.
    Feature values will be written to the Store in the form of FeatureRow elements.
    The way FeatureRow is encoded and decoded when it is written to and read from
    the Store depends on the type of the Store.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    name: builtins.str
    """Name of the store."""
    type: global___StoreType.ValueType
    """Type of store."""
    description: builtins.str
    """Description."""
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        type: global___StoreType.ValueType = ...,
        description: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["description", b"description", "name", b"name", "type", b"type"]) -> None: ...

global___OnlineStore = OnlineStore



from feast.core.OnlineStore_pb2 import OnlineStore as OnlineStoreProto
from feast.core.OnlineStore_pb2 import StoreType


class OnlineStore:
    """
    Database to store features for online serving
    """

    def __init__(
        self,
        name: str,
        store_type: str,
        description: str,
    ):
        self._name = name
        self._description = description
        self._store_type = self._validate_store_type(store_type)

    def __eq__(self, other):
        return self.to_proto() == other.to_proto()

    def __str__(self):
        return str(self.to_proto())

    def __repr__(self):
        return f"OnlineStore <{self.name}>"

    @property
    def name(self):
        """
        Returns the name of this online store
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Sets the name of this online store
        """
        self._name = name

    @property
    def store_type(self):
        """
        Returns the type of this online store
        """
        return self._store_type

    @store_type.setter
    def store_type(self, store_type: str):
        """
        Set the type for this online store
        """
        self._store_type = self._validate_store_type(store_type)

    @property
    def description(self):
        """
        Returns the description of this online store
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Sets the description of this online store
        """
        self._description = description

    @classmethod
    def from_proto(cls, online_store_proto: OnlineStoreProto):
        """
        Creates an OnlineStore from a protobuf representation of an online store
        args:
            online_store_proto: a protobuf representation of an online store
        Returns:
            Returns a OnlineStore object based on the online store protobuf
        """
        online_store = cls(
            name=online_store_proto.name,
            store_type=StoreType.Name(online_store_proto.type),
            description=online_store_proto.description,
        )
        return online_store

    def to_proto(self) -> OnlineStoreProto:
        """
        Converts an OnlineStore object to its protobuf representation.
        Returns:
            OnlineStoreProto protobuf
        """
        online_store_proto = OnlineStoreProto(
            name=self.name,
            type=self.store_type,
            description=self.description,
        )
        return online_store_proto

    @staticmethod
    def _validate_store_type(store_type):
        if store_type in StoreType.keys():
            return store_type
        else:
            raise ValueError(
                f"Unsupported online store type: {store_type}, available store types: {StoreType.keys()}"
            )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/core/Entity.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from feast.types import Value_pb2 as feast_dot_types_dot_Value__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17\x66\x65\x61st/core/Entity.proto\x12\nfeast.core\x1a\x17\x66\x65\x61st/types/Value.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"T\n\x06\x45ntity\x12$\n\x04spec\x18\x01 \x01(\x0b\x32\x16.feast.core.EntitySpec\x12$\n\x04meta\x18\x02 \x01(\x0b\x32\x16.feast.core.EntityMeta\"\xc3\x01\n\nEntitySpec\x12\x0c\n\x04name\x18\x01 \x01(\t\x12/\n\nvalue_type\x18\x02 \x01(\x0e\x32\x1b.feast.types.ValueType.Enum\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\x32\n\x06labels\x18\x08 \x03(\x0b\x32\".feast.core.EntitySpec.LabelsEntry\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x7f\n\nEntityMeta\x12\x35\n\x11\x63reated_timestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12:\n\x16last_updated_timestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampBw\n\x1e\x64\x65v.caraml.store.protobuf.coreB\x0b\x45ntityProtoZHgithub.com/caraml-dev/caraml-store/caraml-store-sdk/go/protos/feast/coreb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'feast.core.Entity_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\036dev.caraml.store.protobuf.coreB\013EntityProtoZHgithub.com/caraml-dev/caraml-store/caraml-store-sdk/go/protos/feast/core'
  _ENTITYSPEC_LABELSENTRY._options = None
  _ENTITYSPEC_LABELSENTRY._serialized_options = b'8\001'
  _ENTITY._serialized_start=97
  _ENTITY._serialized_end=181
  _ENTITYSPEC._serialized_start=184
  _ENTITYSPEC._serialized_end=379
  _ENTITYSPEC_LABELSENTRY._serialized_start=334
  _ENTITYSPEC_LABELSENTRY._serialized_end=379
  _ENTITYMETA._serialized_start=381
  _ENTITYMETA._serialized_end=508
# @@protoc_insertion_point(module_scope)



import re
from setuptools import find_packages, setup

NAME = "caraml-store"
DESCRIPTION = "Python SDK for caraml-store"
URL = "https://github.com/caraml-dev/caraml-store"
AUTHOR = "caraml-dev"
REQUIRES_PYTHON = ">=3.8.0"

REQUIRED = [
    "grpcio>=1.50.0",
    "protobuf>=3.20.0",
    "PyYAML>=6.0.0",
    "croniter==1.*",
]

EXTRA_REQUIRED = {
    "gcp": [
        "pandas>=1.0.0",
        "google-cloud-bigquery>=3.0.0",
    ]
}

# Add Support for parsing tags that have a prefix containing '/' (ie 'sdk/go') to setuptools_scm.
# Regex modified from default tag regex in:
# https://github.com/pypa/setuptools_scm/blob/2a1b46d38fb2b8aeac09853e660bcd0d7c1bc7be/src/setuptools_scm/config.py#L9
TAG_REGEX = re.compile(
    r"^(?:[\/\w-]+)?(?P<version>[vV]?\d+(?:\.\d+){0,2}[^\+]*)(?:\+.*)?$"
)


setup(
    name=NAME,
    author=AUTHOR,
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=find_packages(exclude=("tests",)),
    include_package_data=True,
    install_requires=REQUIRED,
    extras_require=EXTRA_REQUIRED,
    use_scm_version={"root": "../..", "relative_to": __file__, "tag_regex": TAG_REGEX},
    setup_requires=["setuptools_scm"],
    # https://stackoverflow.com/questions/28509965/setuptools-development-requirements
    # Install dev requirements with: pip install -e .[dev]
    license="Apache",
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
    ],
)



from typing import Dict, MutableMapping, Optional

import yaml
from google.protobuf import json_format
from google.protobuf.json_format import MessageToDict, MessageToJson
from google.protobuf.internal.well_known_types import Timestamp

from feast.core.Entity_pb2 import Entity as EntityV2Proto
from feast.core.Entity_pb2 import EntityMeta as EntityMetaProto
from feast.core.Entity_pb2 import EntitySpec as EntitySpecProto
from feast.loaders.file import yaml_loader
from feast.value_type import ValueType


class Entity:
    """
    Represents a collection of entities and associated metadata.
    """

    def __init__(
        self,
        name: str,
        description: str,
        value_type: ValueType,
        labels: Optional[MutableMapping[str, str]] = None,
    ):
        self._name = name
        self._description = description
        self._value_type = value_type
        if labels is None:
            self._labels = dict()  # type: MutableMapping[str, str]
        else:
            self._labels = labels

        self._created_timestamp: Optional[Timestamp] = None
        self._last_updated_timestamp: Optional[Timestamp] = None

    def __eq__(self, other):
        if not isinstance(other, Entity):
            raise TypeError("Comparisons should only involve Entity class objects.")

        if isinstance(self.value_type, int):
            self.value_type = ValueType(self.value_type).name
        if isinstance(other.value_type, int):
            other.value_type = ValueType(other.value_type).name

        if (
            self.labels != other.labels
            or self.name != other.name
            or self.description != other.description
            or self.value_type != other.value_type
        ):
            return False

        return True

    def __str__(self):
        return str(MessageToJson(self.to_proto()))

    @property
    def name(self):
        """
        Returns the name of this entity
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Sets the name of this entity
        """
        self._name = name

    @property
    def description(self):
        """
        Returns the description of this entity
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Sets the description of this entity
        """
        self._description = description

    @property
    def value_type(self):
        """
        Returns the type of this entity
        """
        return self._value_type

    @value_type.setter
    def value_type(self, value_type: ValueType):
        """
        Set the type for this entity
        """
        self._value_type = value_type

    @property
    def labels(self):
        """
        Returns the labels of this entity. This is the user defined metadata
        defined as a dictionary.
        """
        return self._labels

    @labels.setter
    def labels(self, labels: MutableMapping[str, str]):
        """
        Set the labels for this entity
        """
        self._labels = labels

    @property
    def created_timestamp(self):
        """
        Returns the created_timestamp of this entity
        """
        return self._created_timestamp

    @property
    def last_updated_timestamp(self):
        """
        Returns the last_updated_timestamp of this entity
        """
        return self._last_updated_timestamp

    def is_valid(self):
        """
        Validates the state of a entity locally. Raises an exception
        if entity is invalid.
        """

        if not self.name:
            raise ValueError("No name found in entity.")

        if not self.value_type:
            raise ValueError("No type found in entity {self.value_type}")

    @classmethod
    def from_yaml(cls, yml: str):
        """
        Creates an entity from a YAML string body or a file path
        Args:
            yml: Either a file path containing a yaml file or a YAML string
        Returns:
            Returns a EntityV2 object based on the YAML file
        """

        return cls.from_dict(yaml_loader(yml, load_single=True))

    @classmethod
    def from_dict(cls, entity_dict):
        """
        Creates an entity from a dict
        Args:
            entity_dict: A dict representation of an entity
        Returns:
            Returns a EntityV2 object based on the entity dict
        """

        entity_proto = json_format.ParseDict(
            entity_dict, EntityV2Proto(), ignore_unknown_fields=True
        )
        return cls.from_proto(entity_proto)

    @classmethod
    def from_proto(cls, entity_proto: EntityV2Proto):
        """
        Creates an entity from a protobuf representation of an entity
        Args:
            entity_proto: A protobuf representation of an entity
        Returns:
            Returns a EntityV2 object based on the entity protobuf
        """

        entity = cls(
            name=entity_proto.spec.name,
            description=entity_proto.spec.description,
            value_type=ValueType(entity_proto.spec.value_type).name,  # type: ignore
            labels=entity_proto.spec.labels,
        )

        entity._created_timestamp = entity_proto.meta.created_timestamp
        entity._last_updated_timestamp = entity_proto.meta.last_updated_timestamp

        return entity

    def to_proto(self) -> EntityV2Proto:
        """
        Converts an entity object to its protobuf representation
        Returns:
            EntityV2Proto protobuf
        """

        meta = EntityMetaProto(
            created_timestamp=self.created_timestamp,
            last_updated_timestamp=self.last_updated_timestamp,
        )
        if isinstance(self.value_type, ValueType):
            self.value_type = self.value_type.value

        spec = EntitySpecProto(
            name=self.name,
            description=self.description,
            value_type=self.value_type,
            labels=self.labels,
        )

        return EntityV2Proto(spec=spec, meta=meta)

    def to_dict(self) -> Dict:
        """
        Converts entity to dict
        Returns:
            Dictionary object representation of entity
        """

        entity_dict = MessageToDict(self.to_proto())

        # Remove meta when empty for more readable exports
        if entity_dict["meta"] == {}:
            del entity_dict["meta"]

        return entity_dict

    def to_yaml(self):
        """
        Converts a entity to a YAML string.
        Returns:
            Entity string returned in YAML format
        """
        entity_dict = self.to_dict()
        return yaml.dump(entity_dict, allow_unicode=True, sort_keys=False)

    def to_spec_proto(self) -> EntitySpecProto:
        """
        Converts an EntityV2 object to its protobuf representation.
        Used when passing EntitySpecV2 object to Feast request.
        Returns:
            EntitySpecV2 protobuf
        """

        if isinstance(self.value_type, ValueType):
            self.value_type = self.value_type.value

        spec = EntitySpecProto(
            name=self.name,
            description=self.description,
            value_type=self.value_type,
            labels=self.labels,
        )

        return spec

    def _update_from_entity(self, entity):
        """
        Deep replaces one entity with another
        Args:
            entity: Entity to use as a source of configuration
        """

        self.name = entity.name
        self.description = entity.description
        self.value_type = entity.value_type
        self.labels = entity.labels
        self._created_timestamp = entity.created_timestamp
        self._last_updated_timestamp = entity.last_updated_timestamp
